# -*- coding: UTF-8 -*-
# Copyright 2013-2016 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)
"""

demo data specific for :ref:`voga` à la Edmund
"""

from __future__ import unicode_literals

from builtins import range
from builtins import str
import logging
logger = logging.getLogger(__name__)


from lino.utils.instantiator import Instantiator, i2d
from lino.utils import Cycler
from django.utils.translation import ugettext_lazy as _
from django.conf import settings
from atelier.utils import date_offset
from lino.api import dd, rt

DEMO_REF_DATE = i2d(20140101)


cal = dd.resolve_app('cal')
courses = dd.resolve_app('courses')
rooms = dd.resolve_app('rooms')

Booking = dd.resolve_model('rooms.Booking')
Room = dd.resolve_model('cal.Room')
Event = dd.resolve_model('cal.Event')
Partner = dd.resolve_model('contacts.Partner')
Company = dd.resolve_model('contacts.Company')
Teacher = dd.resolve_model('courses.Teacher')
TeacherType = dd.resolve_model('courses.TeacherType')
Pupil = dd.resolve_model('courses.Pupil')
PupilType = dd.resolve_model('courses.PupilType')
Enrolment = dd.resolve_model('courses.Enrolment')
Course = dd.resolve_model('courses.Course')
Product = dd.resolve_model('products.Product')
CourseStates = courses.CourseStates
EnrolmentStates = courses.EnrolmentStates
BookingStates = rooms.BookingStates
Calendar = dd.resolve_model('cal.Calendar')


def demo_date(*args, **kw):
    return date_offset(DEMO_REF_DATE, *args, **kw)


class Loader1(object):

    def objects(self):

        PlaceTypes = rt.models.countries.PlaceTypes
        ProductCat = rt.models.products.ProductCat

        # yield PupilType(ref="M", name="Liige")
        # yield PupilType(ref="H", name="Abistaja")
        # yield PupilType(ref="L", name="Muu")

        yield TeacherType(ref="IN", **dd.str2kw('name', _("Independant")))
        yield TeacherType(ref="VL", **dd.str2kw('name', _("Voluntary")))
        yield TeacherType(ref="ETC", **dd.str2kw('name', _("Other")))

        company = Instantiator('contacts.Company', 'name city:name').build

        tallinn = PlaceTypes.town.find("Tallinn")
        tartu = PlaceTypes.town.find("Tartu")
        rapla = PlaceTypes.town.find("Rapla")
        parnu = PlaceTypes.town.find("Pärnu")

        we = company("Juku õpib MTÜ", tallinn,
                     street="Tartu mnt", street_no=2)
        yield we
        settings.SITE.site_config.site_company = we
        yield settings.SITE.site_config

        # productcat = Instantiator('products.ProductCat').build

        course_fees = ProductCat(**dd.str2kw('name', _("Participation fees")))
        yield course_fees

        trips = ProductCat(**dd.str2kw('name', _("Trips")))
        # et="Väljasõidud", de="Ausflüge", fr="Excursions"))
        yield trips

        kw = dd.str2kw('name', _("Journeys"))
        self.journeys_cat = ProductCat(**kw)
        yield self.journeys_cat

        self.journey_fee = Product(cat=self.journeys_cat, **kw)
        yield self.journey_fee

        rent = ProductCat(**dd.str2kw('name', _("Room renting")))
        # et="Ruumiüür", de="Raummiete", fr="Loyer"))
        yield rent
        # other = ProductCat(**dd.str2kw('name', _("Other")))
        # et="Muud", de="Sonstige", fr="Autres"))
        # yield other

        product = Instantiator(
            'products.Product', "sales_price cat name").build
        yield product("20", course_fees, "20€")
        yield product("50", course_fees, "50€")
        yield product("80", course_fees, "80€")

        rent20 = product("20", rent, "Spiegelraum Eupen")
        yield rent20
        rent10 = product(
            "10", rent, **dd.str2kw('name', _("Rent per meeting")))
        yield rent10

        self.PRICES = Cycler(Product.objects.filter(cat=course_fees))

        event_type = Instantiator('cal.EventType').build
        kw = dd.str2kw('name', _("Courses"))
        kw.update(dd.str2kw('event_label', _("Hour")))
        self.kurse = event_type(**kw)
        yield self.kurse
        settings.SITE.site_config.default_event_type = self.kurse
        yield settings.SITE.site_config

        self.seminare = event_type(**dd.str2kw('name', _("Seminars")))
        yield self.seminare

        yield event_type(**dd.str2kw('name', _("Excursions")))
        yield event_type(**dd.str2kw('name', _("Hikes")))
        yield event_type(**dd.str2kw('name', _("Meetings")))

        yield event_type(
            email_template='Team.eml.html',
            **dd.str2kw('name', _("Team Meetings")))

        company = Instantiator('contacts.Company', 'name city:name').build
        eupen = company("Rüütli õpekeskus", tallinn,
                        street="Rüütli", street_no=39, street_box="/B2")
        yield eupen
        bbach = company("Õpekeskus", tallinn)
        yield bbach
        kelmis = company("Ülikooli", tartu)
        yield kelmis
        stvith = company("Spordi- ja vabaajakeskus", rapla)
        yield stvith

        self.ext1 = company("AA Neudorf", tartu)
        yield self.ext1
        self.ext2 = company("Nisperter Schützenverein", parnu)
        yield self.ext2

        room = Instantiator('cal.Room').build
        kw = dict(company=eupen)
        kw.update(dd.str2kw('name', _("Mirrored room")))
        kw.update(fee=rent20)
        self.spiegel = room(**kw)
        yield self.spiegel

        kw.update(dd.str2kw('name', _("Computer room")))
        kw.update(fee=rent10)
        self.pc_eupen = room(**kw)
        yield self.pc_eupen

        kw = dict(company=bbach)
        kw.update(dd.str2kw('name', _("Conferences room")))
        self.konf = room(**kw)
        yield self.konf

        kw.update(dd.str2kw('name', _("Computer room")))
        self.pc_bbach = room(**kw)
        yield self.pc_bbach

        kw = dict(company=kelmis)
        kw.update(dd.str2kw('name', _("Computer room")))
        self.pc_kelmis = room(**kw)
        yield self.pc_kelmis

        kw = dict(company=stvith)
        kw.update(dd.str2kw('name', _("Computer room")))
        self.pc_stvith = room(**kw)
        yield self.pc_stvith

        COLORS = Cycler(Calendar.COLOR_CHOICES)

        for u in Room.objects.all():
            obj = Calendar(name=str(u), color=COLORS.pop())
            yield obj
            #~ logger.info("20131018 %s", obj)
            u.calendar = obj
            u.save()


class Loader2(Loader1):

    def objects(self):

        yield super(Loader2, self).objects()

        topic = Instantiator('courses.Topic').build
        line = Instantiator('courses.Line', 'topic event_type fee').build
        course = Instantiator(
            'courses.Course', 'line room start_time end_time').build
        booking = Instantiator(
            'rooms.Booking', 'room start_time end_time').build

        TEACHERS = Cycler(Teacher.objects.all())
        COMPANIES = Cycler(Company.objects.all())
        USERS = Cycler(settings.SITE.user_model.objects.all())

        def add_course(*args, **kw):
            kw.update(user=USERS.pop())
            kw.update(teacher=TEACHERS.pop())
            #~ kw.update(price=PRICES.pop())
            return course(*args, **kw)

        Product = rt.models.products.Product
        ProductCat = rt.models.products.ProductCat

        journey_options = ProductCat(**dd.str2kw(
            'name', _("Hotel options")))
        yield journey_options
        option = Instantiator(Product, cat=journey_options).build
        yield option(**dd.str2kw('name', _("Single room")))
        yield option(**dd.str2kw('name', _("Double room")))
        yield option(**dd.str2kw('name', _("Triple room")))
        # yield option(**dd.str2kw('name', _("Shower")))
        # yield option(**dd.str2kw('name', _("Night club")))

        # trip_options = ProductCat(**dd.str2kw('name', _("Trip options")))
        # yield trip_options
        # option = Instantiator(Product, cat=trip_options).build
        # yield option(name="Eupen Oberstadt")
        # yield option(name="Eupen Unterstadt")
        # yield option(name="Raeren")
        # yield option(name="Kelmis")
        # yield option(name="Büllingen")

        journey = Instantiator(
            'courses.Course', 'line name start_date end_date').build

        def add_journey(*args, **kw):
            kw.update(user=USERS.pop())
            kw.update(teacher=TEACHERS.pop())
            return journey(*args, **kw)

        self.journeys_topic = topic(**dd.str2kw('name', _("Journeys")))
        yield self.journeys_topic
        europe = line(self.journeys_topic, None, self.journey_fee,
                      options_cat=journey_options,
                      **dd.str2kw('name', _("Europe")))

        yield europe
        yield add_journey(europe, "Griechenland 2014",
                          i2d(20140814), i2d(20140820))
        yield add_journey(europe, "London 2014",
                          i2d(20140714), i2d(20140720))

        comp = topic(name="Computer")
        yield comp
        sport = topic(name="Sport")
        yield sport
        medit = topic(name="Meditation")
        yield medit
        externe = topic(name="Externe")
        yield externe

        obj = line(comp, self.kurse, self.PRICES.pop(),
                   ref="comp",
                   **dd.str2kw('name', _("First Steps")))
        yield obj
        kw = dict(max_events=8)
        kw.update(max_places=20)
        kw.update(start_date=demo_date(-30))
        kw.update(state=courses.CourseStates.active)
        kw.update(every=1)
        kw.update(every_unit=cal.Recurrencies.weekly)

        yield add_course(obj, self.pc_bbach, "13:30", "15:00",
                         monday=True, **kw)
        yield add_course(obj, self.pc_eupen, "17:30", "19:00",
                         wednesday=True, **kw)
        yield add_course(obj, self.pc_kelmis, "13:30", "15:00",
                         friday=True, **kw)

        desc = """
    Behandelte Themengebiete:

    - Grundlagen, Voraussetzungen
    - Arbeiten im WWW unter Verwendung eines Browsers
    - Navigieren im WWW
    - Links in die Linkleiste legen
    - aus Webseiten heraus drucken
    - Favoriten bzw. Lesezeichen verwenden
    - Aufgabe und Funktionsweise von Suchmaschinen
    - Elektronische Post: E-Mails verfassen, senden, empfangen, beantworten
    - E-Mails mit Anlagen
    - E-mail Sicherheit
    - Tipps und Tricks
    """
        obj = line(
            comp, self.kurse, self.PRICES.pop(),
            ref="WWW",
            description=desc, **dd.str2kw(
                'name', _("Internet for beginners")))
        yield obj
        kw = dict(max_events=8)
        kw.update(start_date=demo_date(10))
        kw.update(state=courses.CourseStates.active)
        yield add_course(obj, self.pc_bbach, "13:30", "15:00",
                         monday=True, **kw)
        yield add_course(obj, self.pc_eupen, "17:30", "19:00",
                         wednesday=True, **kw)
        yield add_course(obj, self.pc_kelmis, "13:30", "15:00",
                         friday=True, **kw)

        obj = line(sport, self.kurse, self.PRICES.pop(),
                   **dd.str2kw('name', _("Belly dancing")))
        yield obj
        kw = dict(max_events=8)
        kw.update(max_places=10)
        kw.update(start_date=demo_date(20))
        kw.update(state=CourseStates.active)
        yield add_course(obj, self.spiegel, "19:00", "20:00",
                         wednesday=True, **kw)

        obj = line(sport, self.kurse, self.PRICES.pop(),
                   **dd.str2kw('name', _("Functional gymnastics")))
        yield obj
        kw = dict(max_events=10, state=CourseStates.active)
        kw.update(start_date=demo_date(-10))
        yield add_course(obj, self.spiegel, "11:00", "12:00", monday=True, **kw)
        yield add_course(obj, self.spiegel, "13:30", "14:30", monday=True, **kw)

        obj = line(sport, self.kurse, self.PRICES.pop(),
                   **dd.str2kw('name', _("Swimming")))
        yield obj
        kw = dict(max_events=10, state=CourseStates.active)
        kw.update(start_date=demo_date(-100))
        yield add_course(obj, self.spiegel, "11:00", "12:00", monday=True, **kw)
        yield add_course(obj, self.spiegel, "13:30", "14:30", monday=True, **kw)
        yield add_course(obj, self.pc_stvith, "11:00", "12:00", tuesday=True, **kw)
        yield add_course(obj, self.pc_stvith, "13:30", "14:30", tuesday=True, **kw)
        yield add_course(obj, self.pc_kelmis, "11:00", "12:00", thursday=True, **kw)
        yield add_course(obj, self.pc_kelmis, "13:30", "14:30", thursday=True, **kw)

        obj = line(sport, self.kurse, self.PRICES.pop(),
                   **dd.str2kw('name', "Self-defence"))
        yield obj
        kw = dict(max_events=6)
        kw.update(max_places=12)
        kw.update(start_date=demo_date(-80))
        kw.update(state=CourseStates.active)
        yield add_course(obj, self.spiegel, "18:00", "19:00", friday=True, **kw)
        yield add_course(obj, self.spiegel, "19:00", "20:00", friday=True, **kw)

        obj = line(medit, self.kurse, self.PRICES.pop(),
                   ref="GLQ",
                   name="GuoLin-Qigong")
        yield obj
        kw = dict(max_events=10)
        kw.update(start_date=demo_date(-10))
        kw.update(state=CourseStates.active)
        yield add_course(obj, self.spiegel, "18:00", "19:30",
                         monday=True, **kw)
        yield add_course(obj, self.spiegel, "19:00", "20:30",
                         friday=True, **kw)

        obj = line(medit, self.kurse, self.PRICES.pop(),
                   ref="MED",
                   **dd.str2kw('name', _("Finding your inner peace")))
        yield obj
        kw = dict(max_events=10)
        kw.update(max_places=30)
        kw.update(start_date=demo_date(-10))
        kw.update(state=CourseStates.active)
        yield add_course(obj, self.konf, "18:00", "19:30", monday=True, **kw)
        yield add_course(obj, self.konf, "19:00", "20:30", friday=True, **kw)

        obj = line(medit, self.kurse, self.PRICES.pop(), name="Yoga")
        yield obj
        kw = dict(max_events=10)
        kw.update(start_date=demo_date(60))
        kw.update(state=CourseStates.active)
        yield add_course(obj, self.konf, "18:00", "19:30", monday=True, **kw)
        yield add_course(obj, self.konf, "19:00", "20:30", friday=True, **kw)

        EXTS = Cycler(self.ext1, self.ext2)

        def add_booking(*args, **kw):
            kw.update(user=USERS.pop())
            kw.update(event_type=self.seminare)
            #~ kw.update(price=PRICES.pop())
            #~ kw.update(tariff=PRICES.pop())
            #~ kw.update(calendar=self.kurse)
            kw.update(every=1)
            kw.update(company=EXTS.pop())
            return booking(*args, **kw)

        #~ obj = line(externe,self.kurse,PRICES.pop(),**dd.babelkw('name',
            #~ de="Raumbuchung",en="Room booking"))
        #~ yield obj
        kw = dict(max_events=10)
        kw.update(every_unit=cal.Recurrencies.weekly)
        kw.update(start_date=demo_date(60))
        kw.update(state=BookingStates.registered)
        kw.update(company=COMPANIES.pop())
        yield add_booking(self.konf, "20:00", "22:00", tuesday=True, **kw)
        kw.update(company=COMPANIES.pop())
        yield add_booking(self.konf, "20:00", "22:00", thursday=True, **kw)

        kw = dict(max_events=1)
        kw.update(every_unit=cal.Recurrencies.once)
        kw.update(company=COMPANIES.pop())
        kw.update(every_unit=cal.Recurrencies.once)
        yield add_booking(self.konf, "10:00", "14:00", **kw)

        PUPILS = Cycler(Pupil.objects.all())
        #~ print 20130712, Pupil.objects.all()
        COURSES = Cycler(Course.objects.filter(line__fee__isnull=False))
        STATES = Cycler(EnrolmentStates.objects())

        for i in range(100):
            kw = dict(
                user=USERS.pop(), course=COURSES.pop(),
                pupil=PUPILS.pop())
            kw.update(request_date=demo_date(-i))
            kw.update(state=STATES.pop())
            #~ print 20130712, kw
            yield Enrolment(**kw)

        #~ ses = settings.SITE.login('rolf')
        ses = settings.SITE.login()

        for model in (Course, Booking):
            for obj in model.objects.all():
                rc = ses.run(obj.do_update_events)
                if not rc.get('success', False):
                    raise Exception("update_reminders on %s returned %s" %
                                    (obj, rc))

        # n = 0
        # for p in Partner.objects.all():
        #     if n > 10:
        #         break
        #     try:
        #         rc = ses.run(p.create_invoice)
        #         #~ print 20130802, rc
        #         if rc.get('success', True):
        #             n += 1
        #     except Warning:
        #         pass


objects = Loader2().objects



# -*- coding: UTF-8 -*-
# Copyright 2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)
"""General demo data for Lino Avanti.

- Course providers and courses

"""

from __future__ import unicode_literals

# from django.conf import settings
# from lino.utils import mti
from lino.utils import Cycler  # join_words
from lino.utils.mldbc import babel_named as named
from lino.api import rt, dd, _

from lino.modlib.users.choicelists import UserTypes
from lino_xl.lib.cal.choicelists import Recurrencies
from lino_xl.lib.courses.choicelists import EnrolmentStates

course_stages = [
    _("Dispens"),
    _("Eingeschrieben"),
    _("Abgeschlossen"),
    _("Abgebrochen"),
    _("Ausgeschlossen")]

trends_config = []
trends_config.append((
    _("Info Integration"),
    [ "!Erstgespräch",
      "Sprachtest",
      "Einschreibung in Sprachkurs",
      "Einschreibung in Integrationskurs",
      "!Bilanzgespräch"]))
trends_config.append((_("Alphabetisation"), course_stages))
trends_config.append((_("A1"), course_stages))
trends_config.append((_("A2"), course_stages))
trends_config.append((_("Citizen course"), course_stages))
trends_config.append((_("Professional integration"), [
    "Begleitet vom DSBE",
    "Begleitet vom ADG",
    "Erwerbstätigkeit",
]))


def objects():

    Line = rt.models.courses.Line
    Teacher = dd.plugins.courses.teacher_model
    Course = rt.models.courses.Course
    Topic = rt.models.courses.Topic
    Enrolment = rt.models.courses.Enrolment
    CourseStates = rt.models.courses.CourseStates
    User = rt.models.users.User
    EventType = rt.models.cal.EventType
    Guest = rt.models.cal.Guest
    GuestRole = rt.models.cal.GuestRole
    GuestStates = rt.models.cal.GuestStates
    EntryStates = rt.models.cal.EntryStates
    Event = rt.models.cal.Event
    Person = rt.models.contacts.Person
    CommentType = rt.models.comments.CommentType
    TrendStage = rt.models.trends.TrendStage
    TrendArea = rt.models.trends.TrendArea

    for area, stages in trends_config:
        ta = named(TrendArea, area)
        yield ta
        for stage in stages:
            kw = dict(trend_area=ta)
            if stage[0] == "!":
                stage = stage[1:]
                kw.update(subject_column=True)
            yield named(TrendStage, stage, **kw)

    yield EventType(**dd.str2kw('name', _("First contact")))

    kw = dd.str2kw('name', _("Lesson"))
    kw.update(dd.str2kw('event_label', _("Lesson")))
    event_type = EventType(**kw)
    yield event_type

    pupil = named(GuestRole, _("Pupil"))
    yield pupil
    yield named(GuestRole, _("Assistant"))

    topic_citizen = named(Topic, _("Citizen course"))
    yield topic_citizen

    topic_lang = named(Topic, _("Language courses"))
    yield topic_lang

    kw.update(topic=topic_citizen)
    kw = dict(event_type=event_type, guest_role=pupil)
    yield named(Line, _("Citizen course"), **kw)

    kw.update(topic=topic_lang)
    alpha = named(Line, _("Alphabetisation"), **kw)
    yield alpha
    yield named(Line, _("German for beginners"), **kw)
    yield named(Line, _("German A1+"), **kw)
    yield named(Line, _("German A2"), **kw)
    yield named(Line, _("German A2 (women)"), **kw)

    yield named(CommentType, _("Phone call"))
    yield named(CommentType, _("Visit"))
    yield named(CommentType, _("Individual consultation"))
    yield named(CommentType, _("Internal meeting"))
    yield named(CommentType, _("Meeting with partners"))

    laura = Teacher(first_name="Laura", last_name="Lieblig")
    yield laura
    yield User(username="laura", user_type=UserTypes.teacher,
               partner=laura)

    yield User(username="nathalie", user_type=UserTypes.user)
    yield User(username="audrey", user_type=UserTypes.auditor)
    yield User(username="martina", user_type=UserTypes.coordinator)
    yield User(username="sandra", user_type=UserTypes.secretary)

    USERS = Cycler(User.objects.exclude(
        user_type__in=(UserTypes.auditor, UserTypes.admin)))

    kw = dict(monday=True, tuesday=True, thursday=True, friday=True)
    kw.update(
        line=alpha,
        start_date=dd.demo_date(-30),
        start_time="9:00", end_time="12:00",
        max_date=dd.demo_date(10),
        state=CourseStates.active,
        every_unit=Recurrencies.daily,
        user=USERS.pop(),
        teacher=laura,
        max_places=5)

    yield Course(**kw)

    kw.update(start_time="14:00", end_time="17:00", user=USERS.pop(),
              max_places=15)
    yield Course(**kw)

    kw.update(start_time="18:00", end_time="20:00", user=USERS.pop(),
              max_places=15)
    yield Course(**kw)


    PUPILS = Cycler(dd.plugins.courses.pupil_model.objects.all())
    # print(20170302, dd.plugins.courses.pupil_model.objects.all())
    COURSES = Cycler(Course.objects.all())
    STATES = Cycler(EnrolmentStates.objects())

    def fits(course, pupil):
        if course.max_places and course.get_free_places() == 0:
            return False
        if Enrolment.objects.filter(course=course, pupil=pupil).count():
            return False
        return True

    def enrol(pupil):
        course = COURSES.pop()
        if fits(course, pupil):
            kw = dict(user=USERS.pop(), course=course, pupil=pupil)
            kw.update(request_date=dd.demo_date(-i))
            kw.update(state=STATES.pop())
            return Enrolment(**kw)

    for i, p in enumerate(
            dd.plugins.courses.pupil_model.objects.order_by('id')):

        yield enrol(p)
        if i % 2 == 0:
            yield enrol(p)
        if i % 3 == 0:
            yield enrol(p)

    ar = rt.login('robin')
    for obj in Course.objects.all():
        obj.update_auto_events(ar)

    # Suggested calendar entries older than 7 days should be marked as
    # either took_place or cancelled.
    qs = Event.objects.filter(
        start_date__lte=dd.demo_date(-7),
        state=EntryStates.suggested)
    for i, obj in enumerate(qs):
        if i % 9:
            obj.state = EntryStates.took_place
        else:
            obj.state = EntryStates.cancelled
        obj.full_clean()
        obj.save()

    # participants of events which took place should be marked as
    # either absent or present or excused:
    qs = Guest.objects.filter(
        event__start_date__lte=dd.demo_date(-7),
        event__state=EntryStates.took_place).order_by('id')
    STATES = Cycler(GuestStates.get_list_items())
    for i, obj in enumerate(qs):
        obj.state = STATES.pop()
        # if i % 8:
        #     obj.state = GuestStates.present
        # elif i % 3:
        #     obj.state = GuestStates.missing
        # else:
        #     obj.state = GuestStates.excused
        obj.full_clean()
        obj.save()



# -*- coding: utf-8 -*-
# Copyright 2016-2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""Runs some tests about the disable-delete handler and cascading deletes.

You can run only these tests by issuing::

  $ go team
  $ python manage.py test tests.test_ddh

Or::

  $ go book
  $ python setup.py test -s tests.ProjectsTests.test_ddh

"""

from __future__ import unicode_literals
from __future__ import print_function

from django.core.exceptions import ValidationError
from lino.utils.djangotest import RemoteAuthTestCase
from lino.api import rt

from lino.utils.instantiator import create_row as create
    

class DDHTests(RemoteAuthTestCase):
    maxDiff = None

    def test01(self):
        from lino.modlib.users.choicelists import UserTypes
        Ticket = rt.models.tickets.Ticket
        Project = rt.models.tickets.Project
        User = rt.models.users.User
        Star = rt.models.votes.Vote
        # ContentType = rt.models.contenttypes.ContentType
        # ct_Ticket = ContentType.objects.get_for_model(Ticket)

        create(Project, name='project')
        robin = create(User, username='robin',
                       first_name="Robin",
                       user_type=UserTypes.admin,
                       language="en")

        def createit():
            return create(Ticket, summary="Test", user=robin)

        #
        # If there are no vetos, user can ask to delete it
        #
        obj = createit()
        obj.delete()

        obj = createit()

        if False:
            try:
                robin.delete()
                self.fail("Expected veto")
            except Warning as e:
                self.assertEqual(
                    str(e), "Cannot delete User Robin "
                    "because 1 Tickets refer to it.")

        
        create(Star, votable=obj, user=robin)
        
        try:
            robin.delete()
            self.fail("Expected veto")
        except Warning as e:
            self.assertEqual(
                str(e), "Cannot delete User Robin "
                "because 1 Tickets refer to it.")

        self.assertEqual(Star.objects.count(), 1)
        self.assertEqual(Ticket.objects.count(), 1)

        obj.delete()
        self.assertEqual(Star.objects.count(), 0)
        self.assertEqual(Ticket.objects.count(), 0)



from lino.api import rt

def objects():
    Country = rt.models.chooser.Country
    City = rt.models.chooser.City

    be = Country(name="Belgium")
    yield be
    yield City(name="Brussels", country=be)
    yield City(name="Eupen", country=be)
    yield City(name="Gent", country=be)

    fr = Country(name="France")
    yield fr
    yield City(name="Paris", country=fr)
    yield City(name="Bordeaux", country=fr)

    ee = Country(name="Estonia")
    yield ee
    yield City(name="Tallinn", country=ee)
    yield City(name="Tartu", country=ee)
    yield City(name="Narva", country=ee)



"""
A fictive application used to satisfy Sphinx autodoc when
generating the API documentation for :ref:`book`.

It contains a settings module to be used as
:envvar:`DJANGO_SETTINGS_MODULE` when Sphinx generates the Lino docs.
See :ref:`lino.dev.bd`.

It installs (almost) all plugins, which makes no sense in practice and
would maybe raise errors if you try to initialize a database or
validate the models, but it is enough to have autodocs do its job.
And that's all we want.


.. autosummary::
   :toctree:


    settings
"""



# -*- coding: UTF-8 -*-
# Copyright 2015-2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""Defines user roles for the Care variant of Lino Noi."""


from lino.api import dd
from lino.core.roles import UserRole, SiteAdmin
from lino.modlib.users.roles import Helper
from lino.modlib.comments.roles import CommentsUser, CommentsStaff
from lino.modlib.office.roles import OfficeStaff, OfficeUser
from lino_xl.lib.contacts.roles import ContactsStaff
from lino_xl.lib.excerpts.roles import ExcerptsUser, ExcerptsStaff
from lino_xl.lib.votes.roles import VotesStaff, VotesUser
from lino_xl.lib.tickets.roles import Reporter, Triager, TicketsStaff
from lino_xl.lib.working.roles import Worker
from lino.modlib.users.choicelists import UserTypes
from django.utils.translation import ugettext_lazy as _


class SimpleUser(Helper, OfficeUser, Reporter, VotesUser,
                 CommentsUser):
    """A **simple user** is a person who can log into the application in
    order to manage their own pleas and competences and potentially
    can respond to other user's pleas.

    """
    pass


class Connector(SimpleUser, Worker, Triager, ExcerptsUser):
    """A **connector** is a person who knows other persons and who
    introduces pleas on their behalf.

    """
    pass


class SiteAdmin(SiteAdmin, OfficeStaff, Helper, ContactsStaff, Worker,
                TicketsStaff, VotesStaff, ExcerptsStaff, CommentsStaff):
    """A **site administrator** can do everything."""


UserTypes.clear()
add = UserTypes.add_item
add('000', _("Anonymous"), UserRole, 'anonymous',
    readonly=not dd.plugins.users.online_registration)
add('100', _("User"), SimpleUser, 'user')
add('500', _("Connector"), Connector, 'connector')
add('900', _("Administrator"),    SiteAdmin, 'admin')



# -*- coding: utf-8 -*-
# Copyright 2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""Test some functionalities-

This module is part of the Lino test suite. You can test only this
module by issuing either::

  $ go mti
  $ python manage.py test
  $ python manage.py test tests.test_misc.QuickTest

or::

  $ go book
  $ python setup.py test -s tests.ProjectsTests.test_mti

"""

from __future__ import unicode_literals
from __future__ import print_function

from django.core.exceptions import ValidationError

from lino.api import dd, rt

from lino.utils.djangotest import RemoteAuthTestCase
from lino.utils.mti import insert_child

class QuickTest(RemoteAuthTestCase):

    # fixtures = ['std', 'demo_users']

    def test_create_entry(self):
        ses = rt.login()

        Person = rt.models.app.Person
        Restaurant = rt.models.app.Restaurant
        Place = rt.models.app.Place
        Visit = rt.models.app.Visit
        Meal = rt.models.app.Meal

        # Create some initial data:

        Person(name="Alfred").save()
        Person(name="Bert").save()
        Person(name="Claude").save()
        Person(name="Dirk").save()
        r = Restaurant(id=1, name="First")
        r.save()
        for i in 1, 2:
            r.owners.add(Person.objects.get(pk=i))
        for i in 3, 4:
            r.cooks.add(Person.objects.get(pk=i))

        # Here is our data:

        lst = list(Person.objects.all())
        self.assertEqual(str(lst), "[<Person: Alfred>, <Person: Bert>, <Person: Claude>, <Person: Dirk>]")

        lst = list(Restaurant.objects.all())
        self.assertEqual(
            str(lst),
            "[Restaurant #1 ('First (owners=Alfred, Bert, cooks=Claude, Dirk)')]")
        
        x = list(Place.objects.all())
        self.assertEqual(
            str(x),
            "[Place #1 ('First (owners=Alfred, Bert)')]")


        """
        The :func:`delete_child` function
        ---------------------------------

        Imagine that a user of our application discovers that Restaurant #1
        isn't actually a `Restaurant`, it's just a `Place`.  They would like
        to "remove it's Restaurant data" from the database, but keep the
        `Place` data.  Especially the primary key (#1) and the related objects
        (the owners) should remain unchanged. But the cooks must be deleted
        since they exist only for restaurants.

        It seems that this is not trivial in Django (`How do you delete child
        class object without deleting parent class object?
        <http://stackoverflow.com/questions/9439730>`__).  That's why we wrote
        the :func:`delete_child` function.
        Here is how to "reduce" a Restaurant to a `Place` by 
        calling the :func:`delete_child` function:

        """

        from lino.utils.mti import delete_child

        p = Place.objects.get(id=1)
        delete_child(p, Restaurant)

        # The Place still exists, but no longer as a Restaurant:

        x = Place.objects.get(pk=1)
        self.assertEqual(
            str(x),
            "First (owners=Alfred, Bert)")

        try:
            list(Restaurant.objects.get(pk=1))
            self.fail("Expected DoesNotExist")
        except Restaurant.DoesNotExist:
            pass
            # Traceback (most recent call last):
        # ...
        # DoesNotExist: Restaurant matching query does not exist.

        """

        The :func:`insert_child` function
        ----------------------------------

        The opposite operation, "promoting a simple Place to a Restaurant", 
        is done using :func:`insert_child`.

        from lino.utils.mti import insert_child

        Let's first create a simple Place #2 with a single owner.
        """

        obj = Place(id=2, name="Second")
        obj.save()
        obj.owners.add(Person.objects.get(pk=2))
        obj.save()
        self.assertEqual(
            str(obj),
            "Second (owners=Bert)")

        # Now this Place becomes a Restaurant and hires 2 cooks:

        obj = insert_child(obj, Restaurant)
        for i in 3, 4:
            obj.cooks.add(Person.objects.get(pk=i))
        self.assertEqual(
            str(obj),
            "Second (owners=Bert, cooks=Claude, Dirk)")

        # If you try to promote a Person to a Restaurant, you'll get an exception:

        person = Person.objects.get(pk=2)
        try:
            insert_child(person, Restaurant).save()
            self.fail("Expected ValidationError")
        except ValidationError as e:
            self.assertEqual(
                str(e), "['A Person cannot be parent for a Restaurant']")

        """
        The :class:`EnableChild` virtual field 
        --------------------------------------

        This section shows how the :class:`EnableChild` virtual field is being 
        used by Lino, and thus is Lino-specific.


        After the above examples our database looks like this:

        """
         
        x = list(Person.objects.all())
        self.assertEqual(
            str(x),
            "[<Person: Alfred>, <Person: Bert>, <Person: Claude>, <Person: Dirk>]")
        x = list(Place.objects.all())
        self.assertEqual(
            str(x),
            "[Place #1 ('First (owners=Alfred, Bert)'), Place #2 ('Second (owners=Bert)')]")
        x = list(Restaurant.objects.all())
        self.assertEqual(
            str(x),
            "[Restaurant #2 ('Second (owners=Bert, cooks=Claude, Dirk)')]")

        # Let's take Place #1 and look at it.

        obj = Place.objects.get(pk=1)
        self.assertEqual(
            str(obj), "First (owners=Alfred, Bert)")

        # How to see whether a given Place is a Restaurant?

        x  = ""
        for i in Place.objects.all():
            x += "{0} -> {1}\n".format(i, i.get_mti_child('restaurant'))
        self.assertEqual(
            x, """\
First (owners=Alfred, Bert) -> None
Second (owners=Bert) -> Second (owners=Bert, cooks=Claude, Dirk)
""")

        # Let's promote First (currently a simple Place) to a Restaurant:

        x = insert_child(obj, Restaurant)
        # Restaurant #1 ('#1 (name=First, owners=Alfred, Bert, cooks=)')


        # And Second stops being a Restaurant:

        second = Place.objects.get(pk=2)
        delete_child(second, Restaurant)

        # This operation has removed the related Restaurant instance:

        try:
            Restaurant.objects.get(pk=2)
            self.fail("Expected DoesNotExist")
        except Restaurant.DoesNotExist:
            pass

        # And finally, rather to explain why Restaurants sometimes 
        # close and later reopen:

        bert = Person.objects.get(pk=2)
        second = Place.objects.get(pk=2)
        insert_child(second, Restaurant)
        # Restaurant #2 ('#2 (name=Second, owners=Bert, cooks=)')

        # Now we can see this place again as a Restaurant

        second = Restaurant.objects.get(pk=2)

        # And engage for example a new cook:

        second.cooks.add(bert)
        # second
        # Restaurant #2 ('#2 (name=Second, owners=Bert, cooks=Bert)')



        # Related objects
        # ---------------

        # Now let's have a more detailed look at what happens to the related 
        # objects (Person, Visit and Meal).

        # Bert, the owner of Restaurant #2 does two visits:

        second = Restaurant.objects.get(pk=2)
        Visit(purpose="Say hello", person=bert, place=second).save()
        Visit(purpose="Hang around", person=bert, place=second).save()
        x = list(second.visit_set.all())
        self.assertEqual(
            str(x),
            "[<Visit: Say hello visit by Bert at Second>, <Visit: Hang around visit by Bert at Second>]")

        # Claude and Dirk, now workless, still go to eat in restaurants:

        Meal(what="Fish",person=Person.objects.get(pk=3),restaurant=second).save()
        Meal(what="Meat",person=Person.objects.get(pk=4),restaurant=second).save()
        x = list(second.meal_set.all())
        self.assertEqual(
            str(x),
            "[<Meal: Claude eats Fish at Second>, <Meal: Dirk eats Meat at Second>]")

        # Now we reduce Second to a Place:

        second = Place.objects.get(pk=2)
        delete_child(second, Restaurant)

        # Restaurant #2 no longer exists:

        try:
            Restaurant.objects.get(pk=2)
            self.fail("Expected DoesNotExist")
        except Restaurant.DoesNotExist:
            pass
        
        # Note that `Meal` has :attr:`allow_cascaded_delete
        # <lino.core.model.Model.allow_cascaded_delete>` set to
        # `['restaurant']`, otherwise the above code would have raised a
        # ValidationError :message:`Cannot delete #2
        # (name=Second,owners=Bert,cooks=Bert) because 2 meals refer to it.` But
        # the meals have been deleted:

        self.assertEqual(Meal.objects.count(), 0)

        # Of course, #2 remains as a Place
        # The owner and visits have been taken over:

        second = Place.objects.get(pk=2)
        x = list(second.visit_set.all())
        self.assertEqual(
            str(x),
            "[<Visit: Say hello visit by Bert at Second>, <Visit: Hang around visit by Bert at Second>]")


        # The :func:`create_mti_child` function
        # -------------------------------------

        # This function is for rather internal use.  :ref:`Python dumps <dpy>`
        # generated by :class:`lino.utils.dpy.Serializer` use this function for
        # creating MTI children instances without having to lookup their parent.

        # .. currentmodule:: lino.utils.dpy

        # In a Python dump we are in a special situation: All Place instances
        # are being generated first, and in another step we are going to create
        # all the Restaurant instances.  So how can we create a Restaurant whose
        # Place already exists *without first having to do a lookup of the Place
        # record*?  That's why :func:`create_mti_child` was written for.

        obj = Place(id=3, name="Third")
        obj.save()
        obj.owners.add(Person.objects.get(pk=2))
        obj.save()
        self.assertEqual(
            str(obj),
            "Third (owners=Bert)")

        from lino.utils.dpy import create_mti_child
        obj = create_mti_child(Place, 3, Restaurant)

        # The return value is technically a normal model instance,
        # but whose `save` and `full_clean` methods have been 
        # patched: `full_clean` is overridden to do nothing, 
        # and `save` will call a "raw" save to avoid the 
        # need of a proper Place instance for that Restaurant.
        # The only thing you can do with it is to save it:

        obj.save()

        # The `save` and `full_clean` methods are the only methods that 
        # will be called by 
        # :class:`lino.utils.dpy.Deserializer`.

        # To test whether :func:`create_mti_child` did her job, 
        # we must re-read an instance:

        obj = Restaurant.objects.get(pk=3)
        self.assertEqual(
            str(obj),
            "Third (owners=Bert, cooks=)")

        # Note that :func:`create_mti_child` supports changing the
        # `name` although that field is defined in the Place model,
        # not in Restaurant. This feature was added 20170626 (#1926,
        # #1923). Before that date Lino raised an exception when you
        # specified a field of the parent model.  And before *that*
        # (until 20120930) this case was silently ignored for
        # backwards compatibility (`/blog/2011/1210`).

        obj = Place(id=4, name="Fourth")
        obj.save()
        ow = create_mti_child(Place, 4, Restaurant, name="A new name")
        ow.full_clean()
        ow.save()
        obj = Restaurant.objects.get(id=4)
        self.assertEqual(obj.name, "A new name")









# -*- coding: UTF-8 -*-
# Copyright 2015-2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)
"""A copy of the :ref:`noi` as a ticket management demo, for testing Django migrations.

.. autosummary::
   :toctree:

   settings

"""




from lino.projects.std.settings import *


class Site(Site):

    # demo_fixtures = ['demo']

    catch_layout_exceptions = False

    def get_installed_apps(self):
        yield super(Site, self).get_installed_apps()
        yield 'lino.modlib.gfks'
        yield 'lino_book.projects.gfktest.lib.gfktest'




# -*- coding: UTF-8 -*-
# Copyright 2009-2020 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

# python setup.py test -s tests.test_misc.PackagesTests

import sys

PY2 = sys.version_info[0] == 2
PY3 = sys.version_info[0] == 3

install_requires = [
    'Sphinx', 'getlino',
    'lino_xl', 'selenium', 'mock',
    'pisa', 'django-wkhtmltopdf',
    'django-iban', 'metafone',
    'djangorestframework',
    # 'bleach',  installed by inv prep
    # 'radicale==1.1.2',
    'radicale',
    'icalendar',
    'vobject',
    'eidreader',   # required to build the book.
    'social-auth-app-django',
    'lino_cosi',
    'lino_noi',
    'lino_voga',
    'lino_welfare',
    'requests_mock',
    'lino_care',
    'lino_vilma',
    'lino_avanti',
    'lino_tera',
    'lino_amici',
    'lino_openui5',
    'commondata', 'commondata.be', 'commondata.ee', 'commondata.eg',
    'mock', 'sqlparse',
]

SETUP_INFO = dict(
    name='lino_book',
    version='20.1.1',
    install_requires=install_requires,
    # dependency_links=[
    #     'git+https://github.com/cylonoven/django-mailbox.git#egg=django_mailbox'],
    description="Lino documentation and demo projects",
    license='BSD-2-Clause',
    include_package_data=True,
    zip_safe=False,
    author='Luc Saffre',
    author_email='luc@lino-framework.org',
    url="http://www.lino-framework.org",
    # ~ test_suite = 'lino_book.projects',
    # test_suite='tests',
    classifiers="""\
  Programming Language :: Python
  Programming Language :: Python :: 2
  Programming Language :: Python :: 3
  Development Status :: 5 - Production/Stable
  Environment :: Web Environment
  Framework :: Django
  Intended Audience :: Developers
  Intended Audience :: System Administrators
  License :: OSI Approved :: BSD License
  Natural Language :: English
  Natural Language :: French
  Natural Language :: German
  Operating System :: OS Independent
  Topic :: Database :: Front-Ends
  Topic :: Home Automation
  Topic :: Office/Business
  Topic :: Software Development :: Libraries :: Application Frameworks""".splitlines())

SETUP_INFO.update(long_description="""

.. image:: https://readthedocs.org/projects/lino/badge/?version=latest
   :target: http://lino.readthedocs.io/en/latest/?badge=latest
.. image:: https://coveralls.io/repos/github/lino-framework/book/badge.svg?branch=master
   :target: https://coveralls.io/github/lino-framework/book?branch=master
.. image:: https://travis-ci.org/lino-framework/book.svg?branch=stable
   :target: https://travis-ci.org/lino-framework/book?branch=stable
.. image:: https://img.shields.io/pypi/v/lino.svg
   :target: https://pypi.python.org/pypi/lino/
.. image:: https://img.shields.io/pypi/l/lino.svg
   :target: https://pypi.python.org/pypi/lino/

This is the code repository that contains (1) the Sphinx source files
of the Lino Book, (2) the ``lino_book`` Python package and (3) a test
suite with doctest-based tests for the Lino framework.

The **Lino Book** is the central documentation tree of the Lino
framework.  It is visible on `www.lino-framework.org
<http://www.lino-framework.org>`__ and on `lino.readthedocs.io
<http://lino.readthedocs.io>`__.

The ``lino_book`` Python package is a collection of small example Lino
applications used for educational and testing purposes.

The code repositories for the ``lino`` and ``lino_xl`` Python packages
have no documentation tree on their own and almost no unit tests, they
are tested and documented here.

Your feedback is welcome.  Our `community page
<http://www.lino-framework.org/community>`__ explains how to contact us.


""")

SETUP_INFO.update(packages=[str(n) for n in """
lino_book
lino_book.projects
lino_book.projects.babel_tutorial
lino_book.projects.babel_tutorial.fixtures
lino_book.projects.dumps
lino_book.projects.dumps.fixtures
lino_book.projects.dumps.settings
lino_book.projects.belref
lino_book.projects.belref.fixtures
lino_book.projects.belref.settings
lino_book.projects.combo
lino_book.projects.combo.fixtures
lino_book.projects.docs
lino_book.projects.docs.settings
lino_book.projects.estref
lino_book.projects.estref.settings
lino_book.projects.estref.tests
lino_book.projects.events
lino_book.projects.events.tests
lino_book.projects.polls
lino_book.projects.polls.polls
lino_book.projects.polls.polls.fixtures
lino_book.projects.polls.mysite
lino_book.projects.polls2
lino_book.projects.polls2.polls
lino_book.projects.polls2.polls.fixtures
lino_book.projects.polls2.mysite
lino_book.projects.polly
lino_book.projects.polly.settings
lino_book.projects.polly.tests
lino_book.projects.20090714
lino_book.projects.20090717
lino_book.projects.20100126
lino_book.projects.20100127
lino_book.projects.20100206
lino_book.projects.20100212
lino_book.projects.20121124
lino_book.projects.chooser
lino_book.projects.chooser.fixtures
lino_book.projects.example
lino_book.projects.properties
lino_book.projects.quantityfield
lino_book.projects.cms
lino_book.projects.cms.settings
lino_book.projects.crl
lino_book.projects.crl.fixtures
lino_book.projects.eric
lino_book.projects.eric.settings
lino_book.projects.eric.settings.fixtures
lino_book.projects.eric.tests
lino_book.projects.homeworkschool
lino_book.projects.homeworkschool.fixtures
lino_book.projects.homeworkschool.settings
lino_book.projects.i18n
lino_book.projects.igen
lino_book.projects.igen.tests
lino_book.projects.max
lino_book.projects.max.settings
lino_book.projects.chatter
lino_book.projects.chatter.settings
lino_book.projects.chatter.tests
lino_book.projects.migs
lino_book.projects.migs.settings
lino_book.projects.migs.settings.fixtures
lino_book.projects.min1
lino_book.projects.min1.settings
lino_book.projects.min1.migrations
lino_book.projects.min1.migrations.about
lino_book.projects.min1.migrations.bootstrap3
lino_book.projects.min1.migrations.contacts
lino_book.projects.min1.migrations.countries
lino_book.projects.min1.migrations.extjs
lino_book.projects.min1.migrations.jinja
lino_book.projects.min1.migrations.lino
lino_book.projects.min1.migrations.office
lino_book.projects.min1.migrations.printing
lino_book.projects.min1.migrations.system
lino_book.projects.min1.migrations.users
lino_book.projects.min1.migrations.xl
lino_book.projects.min2
lino_book.projects.min2.settings
lino_book.projects.min2.tests
lino_book.projects.min3
lino_book.projects.min3.lib
lino_book.projects.min3.lib.contacts
lino_book.projects.min3.lib.contacts.fixtures
lino_book.projects.min3.settings
lino_book.projects.min3.tests
lino_book.projects.min9
lino_book.projects.min9.modlib
lino_book.projects.min9.modlib.contacts
lino_book.projects.min9.modlib.contacts.fixtures
lino_book.projects.min9.modlib.contacts.management
lino_book.projects.min9.modlib.contacts.management.commands
lino_book.projects.min9.settings
lino_book.projects.min9.tests
lino_book.projects.apc
lino_book.projects.apc.settings
lino_book.projects.apc.tests
lino_book.projects.avanti1
lino_book.projects.avanti1.settings
lino_book.projects.avanti1.settings.fixtures
lino_book.projects.avanti1.tests
lino_book.projects.cosi_ee
lino_book.projects.cosi_ee.settings
lino_book.projects.pierre
lino_book.projects.pierre.settings
lino_book.projects.bs3
lino_book.projects.bs3.settings
lino_book.projects.bs3.tests
lino_book.projects.team
lino_book.projects.team.tests
lino_book.projects.team.settings
lino_book.projects.team.settings.fixtures
lino_book.projects.liina
lino_book.projects.liina.tests
lino_book.projects.liina.settings
lino_book.projects.liina.settings.fixtures
lino_book.projects.lydia
lino_book.projects.lydia.tests
lino_book.projects.lydia.settings
lino_book.projects.lydia.settings.fixtures
lino_book.projects.public
lino_book.projects.anna
lino_book.projects.anna.settings
lino_book.projects.anna.tests
lino_book.projects.anna.lib
lino_book.projects.anna.lib.tickets
lino_book.projects.public.settings
lino_book.projects.public.tests
lino_book.projects.lets1
lino_book.projects.lets1.lets
lino_book.projects.lets1.fixtures
lino_book.projects.lets2
lino_book.projects.lets2.lets
lino_book.projects.lets2.fixtures
lino_book.projects.mti
lino_book.projects.mti.app
lino_book.projects.mti.fixtures
lino_book.projects.mti.tests
lino_book.projects.nomti
lino_book.projects.nomti.app
lino_book.projects.nomti.fixtures
lino_book.projects.watch
lino_book.projects.watch.entries
lino_book.projects.watch.fixtures
lino_book.projects.watch.tests
lino_book.projects.watch2
lino_book.projects.watch2.fixtures
lino_book.projects.mldbc
lino_book.projects.mldbc.fixtures
lino_book.projects.actions
lino_book.projects.actors
lino_book.projects.actors.fixtures
lino_book.projects.addrloc
lino_book.projects.addrloc.fixtures
lino_book.projects.myroles
lino_book.projects.auto_create
lino_book.projects.de_BE
lino_book.projects.de_BE.fixtures
lino_book.projects.diamond
lino_book.projects.diamond.main
lino_book.projects.diamond2
lino_book.projects.diamond2.main
lino_book.projects.float2decimal
lino_book.projects.float2decimal.lib
lino_book.projects.float2decimal.lib.float2decimal
lino_book.projects.integer_pk
lino_book.projects.integer_pk.lib
lino_book.projects.integer_pk.lib.integer_pk
lino_book.projects.human
lino_book.projects.gfktest
lino_book.projects.gfktest.settings
lino_book.projects.gfktest.lib
lino_book.projects.gfktest.lib.gfktest
lino_book.projects.pisa
lino_book.projects.sendchanges
lino_book.projects.tables
lino_book.projects.tables.fixtures
lino_book.projects.vtables
lino_book.projects.ovfields
lino_book.projects.edmund
lino_book.projects.edmund.settings
lino_book.projects.edmund.settings.fixtures
lino_book.projects.roger
lino_book.projects.roger.settings
lino_book.projects.roger.settings.fixtures
lino_book.projects.roger.tests
lino_book.projects.workflows
lino_book.projects.workflows.entries
""".splitlines() if n])


SETUP_INFO.update(message_extractors={
    'lino': [
        ('**/sandbox/**',        'ignore', None),
        ('**/cache/**',          'ignore', None),
        ('**.py',                'python', None),
        ('**/linoweb.js',        'jinja2', None),
        #~ ('**.js',                'javascript', None),
        ('**/config/**.html', 'jinja2', None),
        # ~ ('**/templates/**.txt',  'genshi', {
        # ~ 'template_class': 'genshi.template:TextTemplate'
        # ~ })
    ],
})



from lino.api import dd, _
from lino.utils.mldbc.mixins import BabelDesignated


class Expression(BabelDesignated):

    class Meta:
        verbose_name = _('Expression')
        verbose_name_plural = _('Expressions')


class Expressions(dd.Table):
    model = Expression
    column_names = 'id designation *'





# Copyright 2009-2011 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""
Default settings for :mod:`lino.projects.crl`.

"""

import os
import lino

from lino.projects.std.settings import *

from lino.utils.jsgen import js_code


class Site(Site):

    languages = ('en', 'fr', 'de')

    #~ source_dir = os.path.dirname(__file__)
    title = "Lino/CRL"
    #~ domain = "dsbe.saffre-rumma.net"
    help_url = "http://lino.saffre-rumma.net/crl/index.html"

    def is_abstract_model(self, name):
        if name == 'contacts.Person':
            return True
        return False


SITE = Site(globals())



TIME_ZONE = 'Europe/Brussels'

# ~ SITE_ID = 1 # see also fill.py

INSTALLED_APPS = (
    #~ 'django.contrib.auth',
    'lino.modlib.users',
    'lino.modlib.gfks',
    #~ 'django.contrib.sessions',
    #~ 'django.contrib.sites',
    #~ 'django.contrib.markup',
    #~ 'lino.modlib.system',
    'lino',
    'lino_xl.lib.countries',
    #~ 'lino.modlib.documents',
    'lino_xl.lib.properties',
    'lino_xl.lib.contacts',
    #~ 'lino_xl.lib.projects',
    'lino_xl.lib.notes',
    #~ 'lino.modlib.links',
    'lino.modlib.uploads',
    #~ 'lino_xl.lib.thirds',
    'lino_xl.lib.cal',
    #~ 'lino.modlib.jobs',
    'lino.projects.crl',
    # ~ 'south', # http://south.aeracode.org
)



from lino.api import ad
from ..settings import *
from lino.utils import i2d

SITE = Site(globals(),
            the_demo_date=i2d(20141023),
            webdav_protocol='davlink')

ALLOWED_HOSTS = ['*']
DEBUG = True
SECRET_KEY = "20227"  # see :djangoticket:`20227`



# -*- coding: UTF-8 -*-
# Copyright 2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)
"""Shows :ref:`vilma`, a village management tool.

.. autosummary::
   :toctree:

   settings

"""




from lino.projects.std.settings import *


class Site(Site):

    demo_fixtures = ['demo']
    
    def get_installed_apps(self):
        yield super(Site, self).get_installed_apps()
        yield 'lino_book.projects.mti.app'

    def setup_menu(self, profile, main):
        m = main.add_menu("contacts", "Contacts")
        m.add_action('app.Persons')
        m.add_action('app.Places')
        m.add_action('app.Restaurants')

SITE = Site(globals())
DEBUG = True



# -*- coding: UTF-8 -*-
# Copyright 2016-2018 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)
"""The demo fixture for this tutorial.
"""

from datetime import datetime
from django.conf import settings
from django.utils.timezone import make_aware, utc
from lino.api import dd, _
from lino_book.projects.dumps.models import Foo


if settings.USE_TZ:
    def dt(*args):
        # return make_aware(datetime(*args))
        return make_aware(datetime(*args), timezone=utc)
else:
    def dt(*args):
        return datetime(*args)


def objects():
    # three methods for specifying content of babelfields in fixtures:
    yield Foo(
        designation="First", designation_de="Erster", designation_fr="Premier",
        last_visit=dt(2016, 7, 2, 23, 55, 12))
    yield Foo(
        last_visit=dt(2016, 7, 3, 0, 10, 23),
        **dd.str2kw('designation', _("January")))
    yield Foo(
        # last_visit=dt(2016, 10, 30, 4, 34, 0),
        last_visit=dt(2017, 10, 29, 3, 16, 6),
        # last_visit=dt(2012, 10, 28, 4, 34, 0),
        **dd.babelkw('designation', en="Three", de="Drei",
                     fr="Trois", et="Kolm"))






# -*- coding: utf-8 -*-
# Copyright 2018 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""This module contains some quick tests:

You can run only these tests by issuing::

  $ go lydia
  $ python manage.py test tests.test_makecopy

"""

from __future__ import unicode_literals
from __future__ import print_function


from lino.api import rt
from lino.utils.djangotest import RemoteAuthTestCase
from lino.utils.instantiator import create_row
from lino.core.renderer import TestRenderer


#class QuickTest(RemoteAuthTestCase):
class QuickTest(object):  # skipping since MakeCopy is no longer needed.
    maxDiff = None
    # fixtures = ["std", 'demo']
    fixtures = 'std minimal_ledger'.split()
    
    def test_makecopy(self):
        UserTypes = rt.models.users.UserTypes
        Partner = rt.models.contacts.Partner
        Account = rt.models.ledger.Account
        AnaAccountInvoice = rt.models.ana.AnaAccountInvoice
        jnl = rt.models.ledger.Journal.objects.get(ref="PRC")
        # acc = rt.models.ledger.Account.objects.get(ref="PRC")
        user = create_row(rt.models.users.User,
            username="robin", user_type=UserTypes.admin)
        ses = rt.login('robin', renderer=TestRenderer())
        partner = create_row(Partner, name="Foo")
        self.assertEqual(AnaAccountInvoice.objects.count(), 0)
        invoice = jnl.create_voucher(partner=partner, user=user)
        self.assertEqual(
            invoice.__class__, AnaAccountInvoice)
        invoice.full_clean()
        invoice.save()
        ga = Account.objects.filter(
            needs_ana=True, ana_account__isnull=False).order_by('ref')[0]
        for n in range(3):
            i = invoice.add_voucher_item(
                account=ga,
                seqno=3-n,  # enter them in reverse order to reproduce
                            # #2470
                total_incl=123+n)
            i.full_clean()
            i.save()
        
        s = ses.show('ana.ItemsByInvoice', invoice,
                     column_names="seqno account total_incl")
        # print(s)
        expected = """\
==================== ============================= =================
 No.                  Account                       Total incl. VAT
-------------------- ----------------------------- -----------------
 1                    (6010) Purchase of services   125,00
 2                    (6010) Purchase of services   124,00
 3                    (6010) Purchase of services   123,00
 **Total (3 rows)**                                 **372,00**
==================== ============================= =================
"""
        self.assertEqual(s, expected)
        self.assertEqual(AnaAccountInvoice.objects.count(), 1)
        self.assertEqual(invoice.number, 1)

        invoice.make_copy.run_from_session(ses)
        self.assertEqual(AnaAccountInvoice.objects.count(), 2)

        invoice = AnaAccountInvoice.objects.get(number=2)

        s = ses.show('ana.ItemsByInvoice', invoice,
                     column_names="seqno account total_incl")
        # print(s)
        self.assertEqual(s, expected)
        



# -*- coding: UTF-8 -*-
# Copyright 2013-2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""
The `models` module for `lino.projects.min2.modlib.contacts`.
"""

from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _
from django.utils.text import format_lazy

from lino.api import dd, rt

from lino_xl.lib.contacts.models import *
from lino.modlib.comments.mixins import Commentable
from lino_xl.lib.cal.workflows import feedback

# from lino_xl.lib.addresses.mixins import AddressOwner
from lino_xl.lib.dupable_partners.mixins import DupablePartner, DupablePerson


class Partner(Partner, mixins.CreatedModified, DupablePartner, Commentable):
    """A Partner as seen in `lino.projects.min2`.  It does not define any
    specific field but inherits from a specific set of mixins.

    """

    hidden_columns = 'created modified'

    # def get_overview_elems(self, ar):
    #     # In the base classes, Partner must come first because
    #     # otherwise Django won't inherit `meta.verbose_name`. OTOH we
    #     # want to get the `get_overview_elems` from AddressOwner, not
    #     # from Partner (i.e. AddressLocation).
    #     elems = super(Partner, self).get_overview_elems(ar)
    #     elems += AddressOwner.get_overview_elems(self, ar)
    #     return elems


class PartnerDetail(PartnerDetail):

    main = "general contact misc"

    general = dd.Panel("""
    overview:20 general2:20 general3:40
    reception.AppointmentsByPartner comments.CommentsByRFC
    """, label=_("General"))

    general2 = """
    id language
    url
    """

    general3 = """
    email:40
    phone
    gsm
    fax
    """

    contact = dd.Panel("""
    address_box
    remarks:30
    """, label=_("Contact"))

    address_box = """
    country region city zip_code:10
    addr1
    street_prefix street:25 street_no street_box
    addr2
    """

    misc = dd.Panel("""
    created modified
    changes.ChangesByMaster
    """, label=_("Miscellaneous"))


class Person(Person, Partner, DupablePerson):
    """
    Represents a physical person.
    """

    class Meta(Person.Meta):
        verbose_name = _("Person")
        verbose_name_plural = _("Persons")
        #~ ordering = ['last_name','first_name']

    @classmethod
    def get_user_queryset(cls, user):
        qs = super(Person, cls).get_user_queryset(user)
        return qs.select_related('country', 'city')

    def get_print_language(self):
        "Used by DirectPrintAction"
        return self.language

dd.update_field(Person, 'first_name', blank=False)
dd.update_field(Person, 'last_name', blank=False)


class PersonDetail(PersonDetail):

    main = "general contact misc"

    general = dd.Panel("""
    overview:20 general2:40 general3:40
    contacts.RolesByPerson:20 households.MembersByPerson:40 \
    humanlinks.LinksByHuman
    """, label=_("General"))

    general2 = """
    title first_name:15 middle_name:15
    last_name
    gender:10 birth_date age:10
    id language
    """

    general3 = """
    email:40
    phone
    gsm
    fax
    """

    contact = dd.Panel("""
    #address_box addresses.AddressesByPartner
    remarks:30
    """, label=_("Contact"))

    address_box = """
    country region city zip_code:10
    addr1
    street_prefix street:25 street_no street_box
    addr2
    """

    misc = dd.Panel("""
    url
    created modified
    reception.AppointmentsByPartner
    """, label=_("Miscellaneous"))


class Persons(Persons):

    detail_layout = PersonDetail()


class Company(Company, Partner):
    """A company as seen in `lino.projects.min2`.

    .. attribute:: vat_id

        The VAT identification number.

    """
    class Meta:
        verbose_name = _("Organisation")
        verbose_name_plural = _("Organisations")

    vat_id = models.CharField(_("VAT id"), max_length=200, blank=True)


class CompanyDetail(CompanyDetail):

    main = "general contact notes misc"

    general = dd.Panel("""
    overview:20 general2:40 general3:40
    contacts.RolesByCompany
    """, label=_("General"))

    general2 = """
    prefix:20 name:40
    type vat_id
    url
    """

    general3 = """
    email:40
    phone
    gsm
    fax
    """

    contact = dd.Panel("""
    #address_box addresses.AddressesByPartner
    remarks:30
    """, label=_("Contact"))

    address_box = """
    country region city zip_code:10
    addr1
    street_prefix street:25 street_no street_box
    addr2
    """

    notes = "notes.NotesByCompany"

    misc = dd.Panel("""
    id language
    created modified
    reception.AppointmentsByPartner
    """, label=_("Miscellaneous"))


class Companies(Companies):
    detail_layout = CompanyDetail()


# @dd.receiver(dd.post_analyze)
# def my_details(sender, **kw):
#     contacts = sender.modules.contacts

#     contacts.Partners.set_detail_layout(contacts.PartnerDetail())
#     contacts.Companies.set_detail_layout(contacts.CompanyDetail())


Partners.set_detail_layout(PartnerDetail())
Companies.set_detail_layout(CompanyDetail())



# -*- coding: UTF-8 -*-
# Copyright 2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)
"""The :xfile:`settings.py` modules for this variant.

.. autosummary::
   :toctree:

   demo
   fixtures

"""


from lino_vilma.lib.vilma.settings import *


class Site(Site):
    languages = 'en et'
    title = "Lino Vilma"
    

    # def setup_plugins(self):
    #     super(Site, self).setup_plugins()
    #     self.plugins.tickets.configure(
    #         site_model='cal.Room')



from lino_xl.lib.xl.user_types import *

from lino.api import dd, rt
from lino_xl.lib.polls.roles import PollsUser

AllPolls = rt.models.polls.AllPolls
AllPolls.required_roles = dd.login_required(PollsUser)



from lino_book.projects.polls.polls.models import Question, Choice

DATA = """
What is your preferred colour? | Blue | Red | Yellow | other
Do you like Django? | Yes | No | Not yet decided
Do you like ExtJS? | Yes | No | Not yet decided
Which was first? | Checken | Egg | Turkey
"""


def objects():
    for ln in DATA.splitlines():
        if ln:
            a = ln.split('|')
            q = Question(question_text=a[0].strip())
            yield q
            for choice in a[1:]:
                yield Choice(choice_text=choice.strip(), question=q)



# -*- coding: utf-8 -*-
# Copyright 2018 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""
To run only this test::

  $ go combo
  $ python manage.py test

"""

from __future__ import unicode_literals

from django.conf import settings
from django.utils import translation
from django.core.exceptions import ValidationError

from lino.utils.djangotest import RemoteAuthTestCase

from lino.api import dd, rt



class QuickTest(RemoteAuthTestCase):

    def test01(self):
        """
        Tests some basic funtionality.
        """
        self.assertEqual(1+1,3)
        



# -*- coding: UTF-8 -*-
# Copyright 2014-2019 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""
Default settings for a :ref:`cosi` "à la APC".

"""

from __future__ import unicode_literals

from lino_cosi.lib.cosi.settings import *


class Site(Site):
    languages = 'de fr en'
    # demo_fixtures = 'std few_countries minimal_ledger euvatrates \
    # furniture \
    # demo demo_bookings payments demo2'.split()
    demo_fixtures = 'std few_countries minimal_ledger \
    furniture \
    demo demo_bookings payments demo2'.split()

    def get_plugin_configs(self):
        yield super(Site, self).get_plugin_configs()
        yield ('vat', 'declaration_plugin', 'lino_xl.lib.bevat')
        yield ('countries', 'hide_region', True)
        yield ('countries', 'country_code', 'BE')
        yield ('ledger', 'use_pcmn', True)
        yield ('ledger', 'worker_model', 'contacts.Person')



from lino.projects.std.settings import *


class Site(Site):

    verbose_name = "Lino LETS Tutorial (v2)"

    demo_fixtures = ['demo']

    def setup_menu(self, profile, main):
        m = main.add_menu("master", "Master")
        m.add_action(self.models.lets.Members)
        m.add_action(self.models.lets.Customers)
        m.add_action(self.models.lets.Suppliers)
        m.add_action(self.models.lets.Products)

        m = main.add_menu("market", "Market")
        m.add_action(self.models.lets.Offers)
        m.add_action(self.models.lets.Demands)

        m = main.add_menu("config", "Configure")
        m.add_action(self.models.lets.Places)

    def get_dashboard_items(self, user):

        yield self.models.lets.ActiveProducts

SITE = Site(globals(), 'lino_book.projects.lets2.lets')

DEBUG = True




# -*- coding: UTF-8 -*-
# Copyright 2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

from .demo import *
SITE = Site(globals())
# SITE = Site(
#     globals(),
#     remote_user_header='REMOTE_USER')
DEBUG = True
# SITE.appy_params.update(raiseOnError=True)
# SITE.appy_params.update(pythonWithUnoPath='/usr/bin/python3')
# SITE.default_build_method = "appyodt"
# SITE.webdav_url = '/'



# -*- coding: UTF-8 -*-
# Copyright 2012-2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

from lino.projects.std.settings import *


class Site(Site):
    title = "Lino Mini 2"
    demo_fixtures = 'std demo demo2'
    user_types_module = 'lino_xl.lib.xl.user_types'
    workflows_module = 'lino_xl.lib.cal.workflows.feedback'
    use_experimental_features = True

    def setup_quicklinks(self, user, tb):
        super(Site, self).setup_quicklinks(user, tb)
        tb.add_action(self.modules.contacts.Persons)
        tb.add_action(self.modules.contacts.Companies)

    def setup_plugins(self):
        super(Site, self).setup_plugins()
        self.plugins.contacts.configure(use_vcard_export=True)

    def get_installed_apps(self):
        yield super(Site, self).get_installed_apps()

        yield 'lino.modlib.system'
        yield 'lino.modlib.users'
        yield 'lino_xl.lib.contacts'
        yield 'lino_xl.lib.cal'
        yield 'lino.modlib.export_excel'




from django.db.models import Q
from lino.api import dd


class Places(dd.Table):
    model = 'Place'


class Members(dd.Table):
    model = 'Member'
    column_names = "id name place email *"

    detail_layout = """
    id name place email mti_navigator
    """


class Customers(dd.Table):
    model = 'Customer'

    column_names = "id name place email customer_remark *"
    detail_layout = """
    id name place email mti_navigator
    customer_remark
    DemandsByCustomer
    """


class Suppliers(dd.Table):
    model = 'Supplier'

    column_names = "id name place email supplier_remark *"
    detail_layout = """
    id name place email mti_navigator
    supplier_remark
    OffersBySupplier
    """


class Products(dd.Table):
    model = 'Product'
    order_by = ['name']

    detail_layout = """
    id name
    OffersByProduct DemandsByProduct
    """

    column_names = 'id name'


class ActiveProducts(Products):
    
    label = "Active products"
    column_names = 'name offered_by demanded_by'

    @classmethod
    def get_request_queryset(cls, ar):
        # add filter condition to the queryset so that only "active"
        # products are shown, i.e. for which there is at least one
        # offer or one demand.
        qs = super(ActiveProducts, cls).get_request_queryset(ar)
        qs = qs.filter(Q(offer__isnull=False) | Q(demand__isnull=False))
        qs = qs.distinct()
        return qs


class Offers(dd.Table):
    model = 'Offer'
    column_names = "id supplier__name supplier__place " \
                   "supplier__email supplier__supplier_remark *"


class OffersBySupplier(Offers):
    master_key = 'supplier'


class OffersByProduct(Offers):
    master_key = 'product'


class Demands(dd.Table):
    model = 'Demand'


class DemandsByCustomer(Demands):
    master_key = 'customer'


class DemandsByProduct(Demands):
    master_key = 'product'



# Copyright 2008-2013 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)
"""
"""

from django.db import models
from django.utils.translation import ugettext_lazy as _

from lino.api import dd, rt
from lino import mixins



# -*- coding: UTF-8 -*-
# Copyright 2012-2014 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)


import os
import lino

from lino.projects.std.settings import *


class Site(Site):
    #~ title = __name__
    version = "0.0.1"
    verbose_name = "Lino-HWS"
    url = "http://www.lino-framework.org/autodoc/lino.projects.homeworkschool"
    #~ author = "Luc Saffre"
    #~ author_email = "luc.saffre@gmx.net"

    #~ help_url = "http://lino.saffre-rumma.net/az/index.html"

    demo_fixtures = 'std demo demo2'.split()

    #~ project_model = 'contacts.Person'
    #~ project_model = 'courses.Pupil'
    project_model = 'courses.Course'
    #~ project_model = None

    languages = ('en', 'de', 'fr')

    use_eid_jslib = False

    #~ index_view_action = "dsbe.Home"

    override_modlib_models = {
        'contacts.Person': None,
        'sales.Invoice': None,
        'sales.InvoiceItem': None,
    }

    def get_installed_apps(self):
        yield super(Site, self).get_installed_apps()
        yield 'lino.modlib.gfks'
        yield 'lino.modlib.system'
        yield 'lino.modlib.users'
        yield 'lino_xl.lib.countries'
        yield 'lino_xl.lib.contacts'
        yield 'lino_xl.lib.households'
        yield 'lino_xl.lib.notes'
        yield 'lino.modlib.uploads'
        yield 'lino_xl.lib.cal'
        yield 'lino_xl.lib.calview'
        # yield 'lino_xl.lib.extensible'
        yield 'lino_xl.lib.outbox'
        yield 'lino_xl.lib.pages'

        # yield 'lino.modlib.accounts'
        # yield 'lino.modlib.ledger'
        # yield 'lino.modlib.vat'
        yield 'lino_xl.lib.products'
        # yield 'lino.modlib.auto.sales'

        yield 'lino.modlib.courses'
        yield 'lino_book.projects.homeworkschool'



# -*- coding: UTF-8 -*-
# Copyright 2015-2018 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)
"""
A readonly user interface to :mod:`team
<lino_book.projects.team>`.

See :ref:`noi.specs.bs3`.

.. autosummary::
   :toctree:

   settings
   tests
"""




## Copyright 2013-2018 Rumma & Ko Ltd

from lino.projects.std.settings import *

from lino.api import _

class Site(Site):
    
    demo_fixtures = "std demo demo2"
    languages = 'en'
    # user_types_module = 'lino_xl.lib.xl.user_types'
    
    def get_installed_apps(self):
        
        yield super(Site, self).get_installed_apps()
            
        #~ yield 'lino.modlib.gfks'
        # yield 'lino.modlib.system'
        yield 'lino.modlib.users'
        #~ yield 'lino.modlib.changes'
        
        # yield 'lino_xl.lib.countries'
        # yield 'lino_xl.lib.contacts'
        #~ yield 'lino_xl.lib.notes'
        
        yield 'lino_book.projects.watch2'
        
    def setup_menu(self, user_type, main):
        m = main.add_menu("entries", _("Entries"))
        m.add_action(self.models.watch2.Companies)
        m.add_action(self.models.watch2.Entries)
        m.add_action(self.models.watch2.EntryTypes)
        m.add_action(self.models.watch2.CompaniesWithEntryTypes)


SITE = Site(globals())

DEBUG = True



# -*- coding: UTF-8 -*-
# Copyright 2016-2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

from lino_xl.lib.tickets.models import *
from lino.api import _

Ticket.hide_elements('closed')


# class Ticket(Ticket):
#     class Meta(Ticket.Meta):
#         app_label = 'tickets'
#         verbose_name = _("Plea")
#         verbose_name_plural = _("Pleas")
#         abstract = dd.is_abstract_model(__name__, 'Ticket')

# ActiveTickets._label = _("Active pleas")
# UnassignedTickets._label = _("Unassigned pleas")
# PublicTickets._label = _("Public pleas")
# TicketsToTriage._label = _("Pleas to triage")
# TicketsToTalk._label = _("Pleas to talk")
# # TicketsToDo._label = _("Pleas to to")
# AllTickets._label = _("All pleas")

dd.update_field(
    'tickets.Ticket', 'upgrade_notes', verbose_name=_("Solution"))

# dd.update_field(
#     'tickets.Ticket', 'state', default=TicketStates.todo.as_callable)

class TicketDetail(TicketDetail):
    main = "general history_tab more"

    general = dd.Panel("""
    general1:60 votes.VotesByVotable:20 uploads.UploadsByController
    description:30 comments.CommentsByRFC:30 skills.DemandsByDemander #working.SessionsByTicket:20
    """, label=_("General"))

    general1 = """
    summary:40 id:6 deadline
    user:12 end_user:12 #faculty #topic
    site workflow_buttons
    """

    history_tab = dd.Panel("""
    changes.ChangesByMaster:50 #stars.StarsByController:20
    """, label=_("History"), required_roles=dd.login_required(Triager))

    more = dd.Panel("""
    more1:60 #skills.AssignableWorkersByTicket:20
    upgrade_notes LinksByTicket skills.OffersByDemander
    """, label=_("More"), required_roles=dd.login_required(Triager))

    more1 = """
    created modified ticket_type:10
    state priority project
    # standby feedback closed
    """



Tickets.detail_layout = TicketDetail()



# -*- coding: UTF-8 -*-
# Copyright 2015-2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)
"""Database models for this plugin.

"""

from lino.api import dd, _
from lino.utils import join_words
from lino.mixins import  Hierarchical

from lino_xl.lib.contacts.models import *
from lino.modlib.comments.mixins import Commentable
from lino_xl.lib.phones.mixins import ContactDetailsOwner


PartnerDetail.address_box = dd.Panel("""
    name_box
    country #region city zip_code:10
    #addr1
    #street_prefix street:25 street_no street_box
    #addr2
    """, label=_("Address"))

PartnerDetail.contact_box = dd.Panel("""
    url
    phone
    gsm #fax
    """, label=_("Contact"))



class Person(Person, Commentable):
    
    class Meta(Person.Meta):
        app_label = 'contacts'
        abstract = dd.is_abstract_model(__name__, 'Person')
        
    def __str__(self):
        words = []
        words.append(self.first_name)
        words.append(self.last_name)
        return join_words(*words)

    def get_overview_elems(self, ar):
        elems = super(Person, self).get_overview_elems(ar)
        elems += ContactDetailsOwner.get_overview_elems(self, ar)
        return elems

    @classmethod
    def setup_parameters(cls, fields):
        fields.setdefault(
            'company', dd.ForeignKey(
                'contacts.Company', blank=True, null=True))
        super(Person, cls).setup_parameters(fields)
    
    @classmethod
    def get_simple_parameters(cls):
        for p in  super(Person, cls).get_simple_parameters():
            yield p
        yield 'company'
    
    @classmethod
    def add_param_filter(cls, qs, lookup_prefix='', company=None,
                         **kwargs):
        qs = super(Person, cls).add_param_filter(qs, **kwargs)
        if company:
            fkw = dict()
            wanted = company.whole_clan()
            fkw[lookup_prefix + 'rolesbyperson__company__in'] = wanted
            qs = qs.filter(**fkw)
        
        return qs
        

# We use the `overview` field only in detail forms, and we
# don't want it to have a label "Description":
dd.update_field(Person, 'overview', verbose_name=None)    

class Company(Company, Hierarchical, Commentable):
    
    class Meta(Company.Meta):
        app_label = 'contacts'
        abstract = dd.is_abstract_model(__name__, 'Company')
        
    def get_overview_elems(self, ar):
        elems = super(Company, self).get_overview_elems(ar)
        elems += ContactDetailsOwner.get_overview_elems(self, ar)
        return elems



class PersonDetail(PersonDetail):
    
    main = """
    overview contact_box
    contacts.RolesByPerson:30 comments.CommentsByRFC:30
    """

    contact_box = dd.Panel("""
    last_name first_name:15 
    gender #title:10 language:10 
    birth_date age:10 id:6
    """)  #, label=_("Contact"))


class CompaniesByCompany(Companies):
    label = _("Children")
    master_key = 'parent'
    column_names = 'name *'
    
class CompanyDetail(CompanyDetail):
    main = """
    overview:30 general_middle:20 CompaniesByCompany:30
    contacts.RolesByCompany:30 comments.CommentsByRFC:30
    """

    general_middle = """
    type
    parent
    language:10 id:6
    """
    

Companies.set_detail_layout(CompanyDetail())
Persons.set_detail_layout(PersonDetail())
Person.column_names = 'last_name first_name gsm email city *'
Persons.params_layout = 'observed_event start_date end_date company'



from lino.api import rt
from lino.core.diff import ChangeWatcher
from lino.core.utils import PseudoRequest


def objects():

    Company = rt.models.contacts.Company
    
    ar = rt.login(request=PseudoRequest("robin"))

    obj = Company(name="My pub")
    obj.full_clean()
    obj.save_new_instance(ar)
    
    cw = ChangeWatcher(obj)
    obj.name = "Our pub"
    obj.save_watched_instance(ar, cw)

    obj.delete_instance(ar)

    # this is a special fixture : it creates objects as a side effect
    # but does not yield them.
    return []



# -*- coding: UTF-8 -*-
# Copyright 2012-2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

from lino.projects.std.settings import *

import lino_xl


class Site(Site):

    demo_fixtures = 'std few_countries demo demo2'.split()

    verbose_name = "Lino Max"
    version = lino_xl.__version__

    project_name = 'lino_xl_max'

    project_model = 'contacts.Person'
    # use_websockets = True

    # languages = 'en de fr'
    languages = 'en de fr et nl pt-br es'

    user_types_module = 'lino_xl.lib.xl.user_types'

    default_build_method = 'weasy2pdf'

    def get_installed_apps(self):
        yield super(Site, self).get_installed_apps()

        yield 'lino.modlib.system'
        # yield 'lino.modlib.gfks'
        yield 'lino.modlib.users'
        yield 'lino.modlib.changes'
        yield 'lino.modlib.languages'
        yield 'lino_xl.lib.countries'
        yield 'lino_xl.lib.properties'
        yield 'lino_xl.lib.contacts'
        yield 'lino.modlib.checkdata'
        yield 'lino_xl.lib.phones'
        yield 'lino_xl.lib.addresses'
        yield 'lino_xl.lib.humanlinks'  # requires Person to be Born
        yield 'lino_xl.lib.polls'
        yield 'lino_xl.lib.lists'

        yield 'lino.modlib.uploads'
        yield 'lino.modlib.notify'
        yield 'lino_xl.lib.notes'
        yield 'lino_xl.lib.outbox'
        yield 'lino_xl.lib.cal'
        yield 'lino_xl.lib.courses'
        yield 'lino_xl.lib.calview'
        yield 'lino_xl.lib.extensible'
        yield 'lino_xl.lib.reception'
        yield 'lino_xl.lib.excerpts'
        yield 'lino_xl.lib.cv'
        yield 'lino_xl.lib.boards'
        yield 'lino_xl.lib.topics'
        yield 'lino_xl.lib.postings'
        yield 'lino_xl.lib.households'
        yield 'lino_xl.lib.sepa'
        yield 'lino_xl.lib.vat'
        yield 'lino_xl.lib.bevat'
        yield 'lino_xl.lib.sales'
        yield 'lino_xl.lib.invoicing'
        yield 'lino_xl.lib.b2c'
        yield 'lino_xl.lib.deploy'
        yield 'lino_xl.lib.working'
        yield 'lino_xl.lib.skills'
        yield 'lino_xl.lib.tickets'
        yield 'lino_xl.lib.votes'

        yield 'lino_xl.lib.concepts'
        yield 'lino_xl.lib.pages'
        # yield 'lino_xl.lib.beid'
        # yield 'lino.modlib.wkhtmltopdf'
        yield 'lino.modlib.weasyprint'
        yield 'lino_xl.lib.appypod'

        yield 'lino.modlib.tinymce'
        yield 'lino.modlib.export_excel'

        # yield 'lino_welfare.modlib.debts'
        # yield 'lino_welfare.modlib.badges'
        # yield 'lino_welfare.modlib.welfare'
        # yield 'lino_welfare.modlib.pcsw'
        # yield 'lino_xl.lib.b2c'
        # yield 'lino_welfare.modlib.integ'
        # yield 'lino_welfare.modlib.isip'
        # yield 'lino_welfare.modlib.jobs'
        # yield 'lino_welfare.modlib.art61'
        # yield 'lino_welfare.modlib.immersion'
        # yield 'lino_welfare.modlib.active_job_search'
        # yield 'lino_welfare.modlib.xcourses'
        # yield 'lino_welfare.modlib.newcomers'
        # yield 'lino_welfare.modlib.cbss'  # must come after pcsw
        # yield 'lino_welfare.modlib.aids'
        # yield 'lino_welfare.modlib.esf'
        # yield 'lino_welfare.modlib.dupable_clients'


    def get_plugin_configs(self):
        yield super(Site, self).get_plugin_configs()
        yield ('b2c', 'import_statements_path', self.project_dir.child('sepa_in'))
        yield ('countries', 'country_code', 'BE')


    def do_site_startup(self):
        # lino_xl.lib.reception requires some workflow to be imported
        from lino_xl.lib.cal.workflows import feedback
        super(Site, self).do_site_startup()



# -*- coding: UTF-8 -*-
# Copyright 2014-2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)
"""

.. autosummary::
   :toctree:

   doctests
   demo
   www



"""

from lino_riche.lib.riche.settings import *


class Site(Site):
    
    def get_installed_apps(self):
        """Implements :meth:`lino.core.site.Site.get_installed_apps` for Lino
        riche.

        """
        yield super(Site, self).get_installed_apps()
        yield 'lino.modlib.restful'
        # yield 'lino_xl.lib.caldav'



from lino.api import ad

class Plugin(ad.Plugin):
    verbose_name = "Addresses"

    def setup_main_menu(self, site, profile, m):
        m = m.add_menu(self.app_label, self.verbose_name)
        m.add_action('addrloc.Companies')
    



# -*- coding: utf-8 -*-
# Copyright 2016 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""Tests the :manage:`garble_persons` command.

.. You can test only this module by issuing either::

  $ go min2
  $ python manage.py test
  $ python manage.py test tests.test_garble_persons

or::

  $ go book
  $ python setup.py test -s tests.ProjectsTests.test_min2

"""

from __future__ import unicode_literals
from __future__ import print_function

from lino.api import rt

# from lino.modlib.gfks.mixins import Controllable

from lino.utils.djangotest import RemoteAuthTestCase

from django.core.management import call_command


class QuickTest(RemoteAuthTestCase):

    fixtures = ['std', 'few_countries', 'few_cities']

    def test_01(self):
        call_command(
            'garble_persons', '--distribution=ee', '--noinput')



from apiclient.discovery import build
from oauth2client.client import OAuth2Credentials
from django.conf import settings
from datetime import datetime
import httplib2

from lino.api import rt


def doit(social):
    print("User {} authenticated by {}".format(
        social.user, social.provider))
    print(social.extra_data)
    # from social_django.utils import load_strategy
    # social = social.user.social_auth.get(provider='google-oauth2')
    # social = social.user.social_auth.get(provider='google-plus')
    # access_token = social.refresh_token(load_strategy())
    # access_token = social.get_access_token(load_strategy())
    # social.refresh_token(load_strategy())
    # print access_token
    # Get required credentials for the current GooglePlus account
    credentials = OAuth2Credentials(
        social.extra_data['access_token'],
        # access_token,
        settings.SOCIAL_AUTH_GOOGLE_PLUS_KEY,
        settings.SOCIAL_AUTH_GOOGLE_PLUS_SECRET,
        social.extra_data.get('refresh_token', ''),
        datetime.fromtimestamp(social.extra_data['auth_time']),
        None,
        'PythonSocialAuth',
        scopes=settings.SOCIAL_AUTH_GOOGLE_PLUS_SCOPE
    )

    # Get the authorized http object from GooglePlus
    http = httplib2.Http()
    http = credentials.authorize(http)

    # Create the People Service
    people_service = build(serviceName='people', version='v1', http=http)
    # Here is an example to retrieve connections (contacts) of my GooglePlus account In the following line,
    # we are selecting the 'names' and the 'emailAddresses' fields of the contact. To select more fields,
    # please refer to the Google documentation at https://developers.google.com/people/api/rest/v1/people
    # connections = people_service.people().connections().list(
    #     resourceName='people/me',
    #     personFields='names,emailAddresses,phoneNumbers').execute()
    # totalItems = connections.get('totalItems', 0)
    # print (totalItems)
    connections = people_service.people().connections().list(
        resourceName='people/me',
        # pageSize=totalItems,
        pageSize=2000,
        personFields='names,emailAddresses,phoneNumbers').execute()
    print ("User {0} have {1} connections.".format(social.user, len(connections.get('connections', ''))))
    all_contacts = connections.get('connections', [])
    print ("Name            email   ")
    # Showing all the contacts we get.
    for i, contact in enumerate(all_contacts):
        names = contact.get('names', [])
        names = ', '.join([n.get('displayName', '') for n in names])
        s = names
        for fld in ('emailAddresses', 'phoneNumbers'):
            values = []
            for a in contact.get(fld, []):
                value = a.get('value', '')
                _type = a.get('type', '')
                if _type:
                    value = '{0} ({1})'.format(value, _type)
                values.append(value)
            if len(values):
                s += ', ' + ', '.join(values)
        print("[{}] {}".format(i, s))
        # print ("{0}: {1}".format(fld, s))
        # print(contact.keys())


if __name__ == '__main__':
    for sa in rt.models.social_django.UserSocialAuth.objects.filter(
            provider="google-plus"):
        doit(sa)



## Copyright 2013-2018 Rumma & Ko Ltd
## This file is part of the Lino project.

from django.utils.translation import ugettext_lazy as _

from .choicelists import EntryStates
from .actions import WakeupEntry, StartEntry, FinishEntry


EntryStates.new.add_transition(
    _("Reopen"), required_states='done cancelled')
EntryStates.new.add_transition(WakeupEntry)
EntryStates.started.add_transition(StartEntry)
EntryStates.sleeping.add_transition(required_states="new")
EntryStates.done.add_transition(FinishEntry)
EntryStates.cancelled.add_transition(
    required_states='sleeping started',
    help_text=_("""This is a rather verbose help text for the action 
    which triggers transition from 'sleeping' or 'started' 
    to 'cancelled'."""),
    icon_name='cancel')

    



from .models import dd, PartnerType, Partner, Person

class Partners(dd.Table):
    model = Partner
    # no explicit `label` attribute, so take verbose_name_plural from model


class Persons(Partners):
    model = Person
    # no explicit `label` attribute, so take verbose_name_plural from model


class FunnyPersons(Persons):
    label = "Funny persons"


class MyFunnyPersons(FunnyPersons):
    # no explicit `label` attribute, so inherit from parent
    pass


PARTNER_TYPE_CUSTOMER = 1
PARTNER_TYPE_PROVIDER = 2


class TypedPartners(Partners):
    partner_type_pk = None
    
    @classmethod
    def make_instance(cls,**kw):
        kw.update(type_id=cls.partner_type_pk)
        obj = cls.model(**kw)
        return obj

    @classmethod
    def get_request_queryset(cls, ar):
        # override
        qs = super(TypedPartners, cls).get_request_queryset(ar)
        return qs.filter(type__id=cls.partner_type_pk)
        
    @classmethod
    def get_actor_label(cls):
        # override
        if cls.partner_type_pk is None:
            return super(TypedPartners, cls).get_actor_label()
        pt = PartnerType.objects.get(id=cls.partner_type_pk)
        return pt.name


class Customers(TypedPartners):
    partner_type_pk = PARTNER_TYPE_CUSTOMER


class Providers(TypedPartners):
    partner_type_pk = PARTNER_TYPE_PROVIDER



# -*- coding: UTF-8 -*-
# Copyright 2013-2018 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)


from __future__ import unicode_literals
import datetime

from lino.projects.std.settings import *


class Site(Site):

    title = "Lino Events"
    verbose_name = "Lino Events"

    demo_fixtures = 'demo vor'.split()
    the_demo_date = datetime.date(2013, 5, 12)

    languages = 'de fr nl'

    def get_installed_apps(self):
        yield super(Site, self).get_installed_apps()
        yield 'lino.modlib.system'
        yield 'lino_xl.lib.countries'
        yield 'lino_xl.lib.events'

    def get_dashboard_items(self, user):
        from lino.core.dashboard import RequestItem
        for obj in self.models.events.Type.objects.order_by('id'):
            yield RequestItem(obj.EventsByType(
                renderer=self.kernel.default_renderer))
            

SITE = Site(globals())

DEBUG = True



"""A minimal application used by a number of tests and tutorials. It
is not meant to be actually useful in the real world.

It is a little bit less minimal than :mod:`lino.projects.min1` in that
it adds some more modlib plugins:

- :mod:`lino.modlib.changes`
- :mod:`lino_xl.lib.excerpts`
- :mod:`lino_xl.lib.addresses`
- :mod:`lino_xl.lib.reception`
- :mod:`lino.modlib.sepa`
- :mod:`lino_xl.lib.notes`
- :mod:`lino_xl.lib.projects`
- :mod:`lino_xl.lib.humanlinks`
- :mod:`lino_xl.lib.households`
- :mod:`lino_xl.lib.pages`

This is also a **usage example** of :ref:`plugin_inheritance` because if
overrides the :mod:`lino_xl.lib.contacts` plugin by its own version
``lino.projects.min2.modlib.contacts`` (which is not included in this
documentation tree for technical reasons, and anyway you should
inspect the source code if you want to go futher).

The package has a **test suite** for testing some of the plugins it
uses:

.. autosummary::
   :toctree:

   tests.test_addresses
   tests.test_birth_date
   tests.test_min2
   tests.test_cal

"""



# -*- coding: UTF-8 -*-
# Copyright 2015-2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)
"""Shows :ref:`noi` as a ticket management system for software
developer teams.

.. autosummary::
   :toctree:

   settings
   tests
  

.. module:: lino_book.projects.team.urls

Adds customized urlpatters when use_restful is True.

.. data:: urlpatterns

    Blabla.

"""




from polls.models import Question, Choice


def objects():
    p = Question(question_text="What is your preferred colour?")
    yield p
    yield Choice(choice_text="Blue", question=p)
    yield Choice(choice_text="Red", question=p)
    yield Choice(choice_text="Yellow", question=p)
    yield Choice(choice_text="other", question=p)

    p = Question(question_text="Do you like Django?")
    yield p
    yield Choice(choice_text="Yes", question=p)
    yield Choice(choice_text="No", question=p)
    yield Choice(choice_text="Not yet decided", question=p)

    p = Question(question_text="Do you like ExtJS?")
    yield p
    yield Choice(choice_text="Yes", question=p)
    yield Choice(choice_text="No", question=p)
    yield Choice(choice_text="Not yet decided", question=p)




from lino.projects.std.settings import *
# SITE = Site(globals(), 'tables', anonymous_user_type = '900')    
SITE = Site(globals(), 'lino_book.projects.tables')
SITE.demo_fixtures = ['demo']
SITE.site_locale = "en_US.utf-8"

DEBUG = True



"""
Module `lino_xl.lib.properties`
-------------------------------

Imagine that we are doing a study about alimentary habits. We observe a
defined series of properties on the people who participate in our study.
Here are the properties that we are going to observe::


  >>> weight = properties.INT.create_property(name='weight')
  >>> weight.save()

  >>> married = properties.BOOL.create_property(name='married')
  >>> married.save()

  >>> favdish = properties.CHAR.create_property(name='favdish',label='favorite dish')
  >>> favdish.save()
  >>> favdish.create_value("Cookies").save()
  >>> v = favdish.create_value("Fish").save()
  >>> favdish.create_value("Meat").save()
  >>> favdish.create_value("Vegetables").save()


Now we have setup the properties. Let's have a look at this metadata::

  >>> print favdish.choices_list()
  [u'Cookies', u'Fish', u'Meat', u'Vegetables']
  >>> qs = properties.Property.objects.all()
  >>> ["%s (%s)" % (p.name,','.join(map(str,p.choices_list()))) for p in qs]
  [u'weight ()', u'married (True,False)', u'favdish (Cookies,Fish,Meat,Vegetables)']

PropValuesByOwner is a report that cannot be rendered into a normal grid because the 'value' column has variable data type, but it's render_to_dict() method is used to fill an `Ext.grid.PropertyGrid`:

  >>> properties.PropValuesByOwner().request(master=Person).render_to_dict()
  {'count': 3, 'rows': [{'name': u'favdish', 'value': ''}, {'name': u'married', 'value': None}, {'name': u'weight', 'value': None}], 'title': u'Properties for persons'}


Here are the people we are going to analyze::

  >>> chris = Person(name='Chris')
  >>> chris.save()
  >>> fred = Person(name='Fred')
  >>> fred.save()
  >>> vera = Person(name='Vera')
  >>> vera.save()
  >>> mary = Person(name='Mary')
  >>> mary.save()

Now we are ready to fill in some real data. Chris, Fred and Vera
answered together to each question. First we asked them "What's
your weight?", and they answered:

  >>> weight.set_value_for(chris,70)
  >>> weight.set_value_for(fred,110)
  >>> weight.set_value_for(vera,60)

When asked whether they were married, they answered:

  >>> married.set_value_for(chris,True)
  >>> married.set_value_for(fred,False)
  >>> married.set_value_for(vera,True)

And about their favourite dish they answered:

  >>> favdish.set_value_for(chris,'Cookies')
  >>> favdish.set_value_for(fred,'Fish')
  >>> favdish.set_value_for(vera,'Vegetables')

Mary came later. She answered all questions at once, which we can enter
in one line of code:

  >>> properties.set_value_for(mary,married=True,favdish='Meat')

Note that Mary didn't know her weight.

To see the property values of a person, we can use a manual query...

  >>> qs = properties.PropValue.objects.filter(owner_id=fred.pk).order_by('prop__name')
  >>> [v.by_owner() for v in qs]
  [u'favdish: Fish', u'married: False', u'weight: 110']

... or use the `PropValuesByOwner` report:

  >>> properties.PropValuesByOwner().request(master_instance=fred).render_to_dict()
  {'count': 3, 'rows': [{'name': u'favdish', 'value': u'Fish'}, {'name': u'married', 'value': False}, {'name': u'weight', 'value': 110}], 'title': u'Properties for Fred'}

Note how properties.PropValuesByOwner also returns 3 rows for Mary although we don't know her weight:

  >>> properties.PropValuesByOwner().request(master_instance=mary).render_to_dict()
  {'count': 3, 'rows': [{'name': u'favdish', 'value': u'Meat'}, {'name': u'married', 'value': True}, {'name': u'weight', 'value': None}], 'title': u'Properties for Mary'}

Query by property:

  >>> qs = properties.PropValue.objects.filter(prop=weight)
  >>> [v.by_property() for v in qs]
  [u'Chris: 70', u'Fred: 110', u'Vera: 60']

  >>> qs = weight.values_query().order_by('value')
  >>> [v.by_property() for v in qs]
  [u'Vera: 60', u'Chris: 70', u'Fred: 110']


`Report.as_text()`, is currently broken:

  >>> #properties.PropValuesByOwner().as_text(fred)


"""


from django.db import models
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic
from lino_xl.lib.properties import models as properties



class Person(models.Model):
    name = models.CharField(max_length=20)
    properties = generic.GenericRelation(properties.Property)

    def __str__(self):
        return self.name



# -*- coding: UTF-8 -*-
# Copyright 2015-2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

from __future__ import unicode_literals
from __future__ import print_function

import datetime

from lino.api import rt, dd, _
from lino.utils import Cycler, i2d

from lino.core.roles import SiteAdmin
from lino_xl.lib.cal.choicelists import DurationUnits
from lino_xl.lib.working.roles import Worker
from lino.utils.quantities import Duration
from lino.utils.mldbc import babel_named as named
from lino.modlib.users.utils import create_user

from lino_xl.lib.working.choicelists import ReportingTypes

def vote(user, ticket, state, **kw):
    u = rt.models.users.User.objects.get(username=user)
    t = rt.models.tickets.Ticket.objects.get(pk=ticket)
    s = rt.models.votes.VoteStates.get_by_name(state)
    vote = t.get_favourite(u)
    if vote is None:
        vote = rt.models.votes.Vote(user=u, votable=t, state=s, **kw)
    else:
        vote.state = s
    return vote

def objects():
    yield tickets_objects()
    if 'working' in dd.plugins:
        yield working_objects()
    if 'skills' in dd.plugins:
        yield skills_objects()
    yield votes_objects()


def tickets_objects():
    # was previously in tickets
    User = rt.models.users.User
    Company = rt.models.contacts.Company
    Topic = rt.models.topics.Topic
    TT = rt.models.tickets.TicketType
    Ticket = rt.models.tickets.Ticket
    # Competence = rt.models.tickets.Competence
    Interest = rt.models.topics.Interest
    Milestone = dd.plugins.tickets.milestone_model
    # Milestone = rt.models.deploy.Milestone
    if Milestone:
        Deployment = rt.models.deploy.Deployment
        WishTypes = rt.models.deploy.WishTypes
    Project = rt.models.tickets.Project
    # Site = rt.models.tickets.Site
    Site = dd.plugins.tickets.site_model
    Link = rt.models.tickets.Link
    LinkTypes = rt.models.tickets.LinkTypes
    # EntryType = rt.models.blogs.EntryType
    # Entry = rt.models.blogs.Entry
    # Tagging = rt.models.blogs.Tagging
    # Line = rt.models.courses.Line
    # List = rt.models.lists.List
    cons = rt.models.users.UserTypes.consultant
    dev = rt.models.users.UserTypes.developer
    yield create_user("marc", rt.models.users.UserTypes.user)
    yield create_user("mathieu", rt.models.users.UserTypes.user)
    yield create_user("luc", dev)
    yield create_user("jean", rt.models.users.UserTypes.senior)

    USERS = Cycler(User.objects.all())
    WORKERS = Cycler(User.objects.filter(
        username__in='luc jean'.split()))
    END_USERS = Cycler(User.objects.filter(user_type=''))

    yield named(TT, _("Bugfix"))
    yield named(TT, _("Enhancement"))
    yield named(TT, _("Upgrade"))
    
    # sprint = named(Line, _("Sprint"))
    # yield sprint

    TYPES = Cycler(TT.objects.all())

    yield Topic(name="Lino Core", ref="linõ")
    yield Topic(name="Lino Welfare", ref="welfäre")
    yield Topic(name="Lino Cosi", ref="così")
    yield Topic(name="Lino Voga", ref="faggio")
    # ref differs from name

    TOPICS = Cycler(Topic.objects.all())

    for name in "Bus.co farm.co share.co".split():

        obj = Company(name=name)
        yield obj
        yield Site(name=name + ".com", partner=obj)

    COMPANIES = Cycler(Company.objects.all())
    
    yield Company(name="in3x")
    
    for u in Company.objects.exclude(name="in3x"):
        for i in range(3):
            yield Interest(owner=u, topic=TOPICS.pop())

    roletype = rt.models.contacts.RoleType
    yield roletype(**dd.babel_values('name', en="Manager", fr='Gérant', de="Geschäftsführer", et="Tegevjuht"))
    yield roletype(**dd.babel_values('name', en="Director", fr='Directeur', de="Direktor", et="Direktor"))
    yield roletype(**dd.babel_values('name', en="Secretary", fr='Secrétaire', de="Sekretär", et="Sekretär"))
    yield roletype(**dd.babel_values('name', en="IT Manager", fr='Gérant informatique', de="EDV-Manager", et="IT manager"))
    yield roletype(**dd.babel_values('name', en="President", fr='Président', de="Präsident", et="President"))


    # RTYPES = Cycler(ReportingTypes.objects())
    
    prj1 = Project(
        name="Framewörk", ref="linö", private=False,
        company=COMPANIES.pop(),
        # reporting_type=RTYPES.pop(),
        start_date=i2d(20090101))
    yield prj1
    yield Project(
        name="Téam", ref="téam", start_date=i2d(20100101),
        # reporting_type=RTYPES.pop(),
        company=COMPANIES.pop(),
        parent=prj1, private=True)
    prj2 = Project(
        name="Documentatión", ref="docs", private=False,
        # reporting_type=RTYPES.pop(),
        company=COMPANIES.pop(),
        start_date=i2d(20090101), parent=prj1)
    yield prj2
    yield Project(
        name="Research", ref="research", private=False,
        company=COMPANIES.pop(),
        start_date=i2d(19980101), parent=prj2)
    yield Project(
        name="Shop", ref="shop", private=False,
        # reporting_type=RTYPES.pop(),
        company=COMPANIES.pop(),
        start_date=i2d(20120201), end_date=i2d(20120630))

    PROJECTS = Cycler(Project.objects.all())

    # for u in User.objects.all():
    #     yield Competence(user=u, project=PROJECTS.pop())
    #     yield Competence(user=u, project=PROJECTS.pop())
    
    SITES = Cycler(Site.objects.exclude(name="pypi"))
    # LISTS = Cycler(List.objects.all())

    if Milestone:
        for i in range(7):
            site = SITES.pop()
            d = dd.today(i*2-20)
            kw = dict(
                user=WORKERS.pop(),
                start_date=d,
                # line=sprint,
                # project=PROJECTS.pop(), # expected=d, reached=d,
                # expected=d, reached=d,
                name="{}@{}".format(d.strftime("%Y%m%d"), site),
                # list=LISTS.pop()
            )
            kw[Milestone.site_field_name] = site
            yield Milestone(**kw)
        # yield Milestone(site=SITES.pop(), expected=dd.today())
        # yield Milestone(project=PROJECTS.pop(), expected=dd.today())
    
    SITES = Cycler(Site.objects.all())
    
    TicketStates = rt.models.tickets.TicketStates
    TSTATES = Cycler(TicketStates.objects())
    
    Vote = rt.models.votes.Vote
    VoteStates = rt.models.votes.VoteStates
    VSTATES = Cycler(VoteStates.objects())

    num = [0]
    
    def ticket(summary, **kwargs):
        num[0] += 1
        u = WORKERS.pop()
        kwargs.update(
            ticket_type=TYPES.pop(), summary=summary,
            user=u,
            state=TSTATES.pop(),
            topic=TOPICS.pop())
        if num[0] % 2:
            kwargs.update(site=SITES.pop())
        if num[0] % 4:
            kwargs.update(private=True)
        if num[0] % 5:
            kwargs.update(end_user=END_USERS.pop())
        if False:
            kwargs.update(project=PROJECTS.pop())
        obj = Ticket(**kwargs)
        yield obj
        if obj.state.active:
            yield Vote(
                votable=obj, user=WORKERS.pop(), state=VSTATES.pop())

    yield ticket(
        "Föö fails to bar when baz", project=PROJECTS.pop())
    yield ticket("Bar is not always baz", project=PROJECTS.pop())
    yield ticket("Baz sucks")
    yield ticket("Foo and bar don't baz", project=PROJECTS.pop())
    # a ticket without project:
    yield ticket("Cannot create Foo", description="""<p>When I try to create
    a <b>Foo</b>, then I get a <b>Bar</b> instead of a Foo.</p>""")

    yield ticket("Sell bar in baz", project=PROJECTS.pop())
    yield ticket("No Foo after deleting Bar", project=PROJECTS.pop())
    yield ticket("Is there any Bar in Foo?", project=PROJECTS.pop())
    yield ticket("Foo never matches Bar", project=PROJECTS.pop())
    yield ticket("Where can I find a Foo when bazing Bazes?",
                 project=PROJECTS.pop())
    yield ticket("Class-based Foos and Bars?", project=PROJECTS.pop())
    yield ticket("Foo cannot bar", project=PROJECTS.pop())

    # Example of memo markup:
    yield ticket("Bar cannot foo", project=PROJECTS.pop(),
                 description="""<p>Linking to [ticket 1] and to
                 [url http://luc.lino-framework.org/blog/2015/0923.html blog].</p>
                 """)
 
    yield ticket("Bar cannot baz", project=PROJECTS.pop())
    yield ticket("Bars have no foo", project=PROJECTS.pop())
    yield ticket("How to get bar from foo", project=PROJECTS.pop())

    n = Ticket.objects.count()

    for i in range(100):
        yield ticket("Ticket {}".format(i+n+1), project=PROJECTS.pop())

    if Milestone:
        WTYPES = Cycler(WishTypes.objects())
        MILESTONES = Cycler(Milestone.objects.all())
        for t in Ticket.objects.all():
            t.set_author_votes()
            if t.id % 4:
                yield Deployment(
                    milestone=MILESTONES.pop(), ticket=t,
                    wish_type=WTYPES.pop())

    
    yield Link(
        type=LinkTypes.requires,
        parent=Ticket.objects.get(pk=1),
        child=Ticket.objects.get(pk=2))

    # yield EntryType(**dd.str2kw('name', _('Release note')))
    # yield EntryType(**dd.str2kw('name', _('Feature')))
    # yield EntryType(**dd.str2kw('name', _('Upgrade instruction')))

    # ETYPES = Cycler(EntryType.objects.all())
    # TIMES = Cycler('12:34', '8:30', '3:45', '6:02')
    # blogger = USERS.pop()
    #
    # def entry(offset, title, body, **kwargs):
    #     kwargs['user'] = blogger
    #     kwargs['entry_type'] = ETYPES.pop()
    #     kwargs['pub_date'] = dd.today(offset)
    #     kwargs['pub_time'] = TIMES.pop()
    #     return Entry(title=title, body=body, **kwargs)
    #
    # yield entry(-3, "Hello, world!", "This is our first blog entry.")
    # e = entry(-2, "Hello again", "Our second blog entry is about [ticket 1]")
    # yield e
    # yield Interest(owner=e, topic=TOPICS.pop())
    #
    # e = entry(-1, "Our third entry", """\
    # Yet another blog entry about [ticket 1] and [ticket 2].
    # This entry has two taggings""")
    # yield e
    # yield Interest(owner=e, topic=TOPICS.pop())
    # yield Interest(owner=e, topic=TOPICS.pop())

def working_objects():
    # was previously in working
    Company = rt.models.contacts.Company
    Vote = rt.models.votes.Vote
    SessionType = rt.models.working.SessionType
    Session = rt.models.working.Session
    Ticket = rt.models.tickets.Ticket
    User = rt.models.users.User
    UserTypes = rt.models.users.UserTypes
    # devs = (UserTypes.developer, UserTypes.senior)
    devs = [p for p in UserTypes.items()
            if p.has_required_roles([Worker])
            and not p.has_required_roles([SiteAdmin])]
    workers = User.objects.filter(user_type__in=devs)
    # WORKERS = Cycler(workers)
    TYPES = Cycler(SessionType.objects.all())
    # TICKETS = Cycler(Ticket.objects.all())
    DURATIONS = Cycler([12, 138, 90, 10, 122, 209, 37, 62, 179, 233, 5])

    # every fourth ticket is unassigned and thus listed in
    # PublicTickets
    # for i, t in enumerate(Ticket.objects.exclude(private=True)):
    # for i, t in enumerate(Ticket.objects.all()):
    #     if i % 4:
    #         t.assigned_to = WORKERS.pop()
    #         yield t

    for u in workers:

        VOTES = Cycler(Vote.objects.filter(user=u))
        # TICKETS = Cycler(Ticket.objects.filter(assigned_to=u))
        if len(VOTES) == 0:
            continue

        for offset in (0, -1, -3, -4):

            date = dd.demo_date(offset)
            worked = Duration()
            ts = datetime.datetime.combine(date, datetime.time(9, 0, 0))
            for i in range(7):
                obj = Session(
                    ticket=VOTES.pop().votable,
                    session_type=TYPES.pop(), user=u)
                obj.set_datetime('start', ts)
                d = DURATIONS.pop()
                worked += d
                if offset < 0:
                    ts = DurationUnits.minutes.add_duration(ts, d)
                    obj.set_datetime('end', ts)
                yield obj
                if offset == 0 or worked > 8:
                    break

    ServiceReport = rt.models.working.ServiceReport
    welket = Company.objects.get(name="welket")
    yield ServiceReport(
        start_date=dd.today(-90), interesting_for=welket)


def skills_objects():
    "was previously in skills.fixtures.demo2"

    Skill = rt.models.skills.Skill
    Competence = rt.models.skills.Competence
    Demand = rt.models.skills.Demand
    # Ticket = rt.models.tickets.Ticket
    User = rt.models.users.User

    yield named(Skill, _('Analysis'))
    yield named(Skill, _('Code changes'))
    yield named(Skill, _('Documentation'))
    yield named(Skill, _('Testing'))
    yield named(Skill, _('Configuration'))
    yield named(Skill, _('Enhancement'))
    yield named(Skill, _('Optimization'))
    yield named(Skill, _('Offer'))

    SKILLS = Cycler(Skill.objects.all())
    END_USERS = Cycler(dd.plugins.skills.end_user_model.objects.all())

    i = 0
    for j in range(2):
        for u in User.objects.all():
            i += 1
            yield Competence(user=u, faculty=SKILLS.pop())
            if i % 2:
                yield Competence(user=u, faculty=SKILLS.pop())
            if i % 3:
                yield Competence(
                    user=u, faculty=SKILLS.pop(),
                    end_user=END_USERS.pop())
            
    for i, t in enumerate(
            dd.plugins.skills.demander_model.objects.all()):
        yield Demand(demander=t, skill=SKILLS.pop())
        if i % 3:
            yield Demand(demander=t, skill=SKILLS.pop())


def votes_objects():

    yield vote('jean', 1, 'candidate')
    yield vote('luc', 1, 'candidate')
    yield vote('jean', 2, 'assigned')



from django.db import models
from lino.api import dd
from django.utils.translation import ugettext_lazy as _

from lino.modlib.users.mixins import My, UserAuthored


class Entry(UserAuthored):

    class Meta:
        verbose_name = _("Entry")
        verbose_name_plural = _("Entries")

    subject = models.CharField(_("Subject"), blank=True, max_length=200)
    body = dd.RichTextField(_("Body"), blank=True)
    company = dd.ForeignKey('contacts.Company')


class Entries(dd.Table):
    model = Entry

    detail_layout = """
    id user
    company
    subject
    body
    """

    insert_layout = """
    company
    subject
    """


class EntriesByCompany(Entries):
    master_key = 'company'


class MyEntries(My, Entries):
    pass


@dd.receiver(dd.post_startup)
def my_change_watchers(sender, **kw):
    """
    This site watches the changes to Partner, Company and Entry
    """
    self = sender
    
    from lino.utils.watch import watch_changes as wc
    
    # In our example we want to collect changes to Company and Entry
    # objects to their respective Partner.

    wc(self.models.contacts.Partner)
    wc(self.models.contacts.Company, master_key='partner_ptr')
    wc(self.models.entries.Entry, master_key='company__partner_ptr')

    # add two application-specific panels, one to Partners, one to
    # Companies:
    
    self.models.contacts.Companies.add_detail_tab(
        'changes', 'changes.ChangesByMaster')
    self.models.contacts.Companies.add_detail_tab(
        'entries', 'entries.EntriesByCompany')




# -*- coding: UTF-8 -*-
# Copyright 2014 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""
The :xfile:`models` module for :ref:`estref` app.

"""

from lino.api import dd, rt
from lino.core.permissions import SiteUser, StaffMember


def site_setup(site):
    lst = (site.modules.countries.Places,
           site.modules.countries.Countries)
    for t in lst:
        t.required_roles.discard(SiteUser)
        t.required_roles.discard(StaffMember)
